@@ -23,13 +23,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
- * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Revision: 24853 $
+ * $Author: csoutheren $
+ * $Date: 2010-11-01 09:33:59 -0500 (Mon, 01 Nov 2010) $
  */
 
-#ifndef _PSNMP_H
-#define _PSNMP_H
+#ifndef PTLIB_PSNMP_H
+#define PTLIB_PSNMP_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -170,7 +170,8 @@ class PSNMP : public PIndirectChannel
                       const PSNMPVarBindingList & vars,
                        const PIPSocket::Address & agentAddress);
 
-    static PBoolean DecodeTrap(const PBYTEArray & readBuffer,
+/*
+  static PBoolean DecodeTrap(const PBYTEArray & readBuffer,
                                        PINDEX & version,
                                       PString & community,
                                       PString & enterprise,
@@ -179,6 +180,7 @@ class PSNMP : public PIndirectChannel
                                       PINDEX  & specificTrapType,
                                  PASNUnsigned & timeTicks,
                           PSNMPVarBindingList & varsOut);
+*/
 
 	typedef list<pair<PString,PRFC1155_ObjectSyntax> > BindingList;
 };
@@ -295,7 +297,7 @@ class PSNMPServer : public PSNMP, PThread
 
 #endif // P_SNMP
 
-#endif
+#endif // PTLIB_PSNMP_H
 
 
-// End of File.
+// End Of File ///////////////////////////////////////////////////////////////
