@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22848 $
+ * $Revision: 25892 $
  * $Author: rjongbloed $
- * $Date: 2009-06-12 06:26:03 +0000 (Fri, 12 Jun 2009) $
+ * $Date: 2011-05-25 20:27:19 -0500 (Wed, 25 May 2011) $
  */
 
 #ifndef PTLIB_VXML_H
@@ -38,6 +38,7 @@
 
 #if P_VXML
 
+#include <ptlib/pfactory.h>
 #include <ptlib/pipechan.h>
 #include <ptclib/delaychan.h>
 #include <ptclib/pwavfile.h>
@@ -56,14 +57,15 @@ class PVXMLSession;
 #define VXML_G7231         "G.723.1"
 #define VXML_G729          "G.729"
 
-#define PVXML_HAS_FACTORY   1
+
+//////////////////////////////////////////////////////////////////
 
 class PVXMLGrammar : public PObject
 {
   PCLASSINFO(PVXMLGrammar, PObject);
   public:
     PVXMLGrammar(PXMLElement * field);
-    virtual PBoolean OnUserInput(const char /*ch*/) { return PTrue; }
+    virtual PBoolean OnUserInput(const char /*ch*/) { return true; }
     virtual void Stop() { }
 
     PString GetValue() const { return value; }
@@ -151,7 +153,6 @@ class PVXMLChannel;
 class PVXMLChannelInterface {
   public:
     virtual ~PVXMLChannelInterface() { }
-    virtual PWAVFile * CreateWAVFile(const PFilePath & fn, PFile::OpenMode mode, int opts, unsigned fmt) = 0;
     virtual void RecordEnd() = 0;
     virtual void OnEndRecording(const PString & channelName) = 0;
     virtual void Trigger() = 0;
@@ -163,13 +164,13 @@ class PVXMLSession : public PIndirectChannel, public PVXMLChannelInterface
 {
   PCLASSINFO(PVXMLSession, PIndirectChannel);
   public:
-    PVXMLSession(PTextToSpeech * tts = NULL, PBoolean autoDelete = PFalse);
+    PVXMLSession(PTextToSpeech * tts = NULL, PBoolean autoDelete = false);
     virtual ~PVXMLSession();
 
     // new functions
     PTextToSpeech * SetTextToSpeech(PTextToSpeech * tts, PBoolean autoDelete = false);
     PTextToSpeech * SetTextToSpeech(const PString & ttsName);
-    PTextToSpeech * GetTextToSpeech() { return textToSpeech; }
+    PTextToSpeech * GetTextToSpeech() const { return textToSpeech; }
 
     virtual PBoolean Load(const PString & source);
     virtual PBoolean LoadFile(const PFilePath & file);
@@ -180,25 +181,18 @@ class PVXMLSession : public PIndirectChannel, public PVXMLChannelInterface
     virtual PBoolean Open(const PString & mediaFormat);
     virtual PBoolean Close();
 
-    PBoolean Execute();
+    virtual PBoolean Execute();
 
-    PVXMLChannel * GetAndLockVXMLChannel() 
-    { 
-      sessionMutex.Wait(); 
-      if (vxmlChannel != NULL) 
-        return vxmlChannel; 
-      sessionMutex.Signal();
-      return NULL;
-    }
+    PVXMLChannel * GetAndLockVXMLChannel();
     void UnLockVXMLChannel() { sessionMutex.Signal(); }
     PMutex & GetSessionMutex() { return sessionMutex; }
 
-    PBoolean LoadGrammar(PVXMLGrammar * grammar);
+    virtual PBoolean LoadGrammar(PVXMLGrammar * grammar);
 
     virtual PBoolean PlayText(const PString & text, PTextToSpeech::TextType type = PTextToSpeech::Default, PINDEX repeat = 1, PINDEX delay = 0);
     PBoolean ConvertTextToFilenameList(const PString & text, PTextToSpeech::TextType type, PStringArray & list, PBoolean useCacheing);
 
-    virtual PBoolean PlayFile(const PString & fn, PINDEX repeat = 1, PINDEX delay = 0, PBoolean autoDelete = PFalse);
+    virtual PBoolean PlayFile(const PString & fn, PINDEX repeat = 1, PINDEX delay = 0, PBoolean autoDelete = false);
     virtual PBoolean PlayData(const PBYTEArray & data, PINDEX repeat = 1, PINDEX delay = 0);
     virtual PBoolean PlayCommand(const PString & data, PINDEX repeat = 1, PINDEX delay = 0);
     virtual PBoolean PlayResource(const PURL & url, PINDEX repeat = 1, PINDEX delay = 0);
@@ -222,13 +216,16 @@ class PVXMLSession : public PIndirectChannel, public PVXMLChannelInterface
 
     PString GetXMLError() const;
 
+    virtual void OnEndDialog();
     virtual void OnEndSession()         { }
+    virtual void OnTransfer(const PString & /*destination*/, bool /*bridged*/) { }
 
-    virtual PString GetVar(const PString & str) const;
+    const PStringToString & GetVariables() { return m_variables; }
+    virtual PCaselessString GetVar(const PString & str) const;
     virtual void SetVar(const PString & ostr, const PString & val);
     virtual PString EvaluateExpr(const PString & oexpr);
 
-    virtual PBoolean RetreiveResource(const PURL & url, PString & contentType, PFilePath & fn, PBoolean useCache = PTrue);
+    virtual PBoolean RetreiveResource(const PURL & url, PString & contentType, PFilePath & fn, PBoolean useCache = true);
 
     PDECLARE_NOTIFIER(PThread, PVXMLSession, VXMLExecute);
 
@@ -237,33 +234,39 @@ class PVXMLSession : public PIndirectChannel, public PVXMLChannelInterface
     PXMLElement * FindHandler(const PString & event);
 
     // overrides from VXMLChannelInterface
-    PWAVFile * CreateWAVFile(const PFilePath & fn, PFile::OpenMode mode, int opts, unsigned fmt);
-    void OnEndRecording(const PString & channelName);
-    void RecordEnd();
-    void Trigger();
+    virtual void OnEndRecording(const PString & channelName);
+    virtual void RecordEnd();
+    virtual void Trigger();
 
-    PStringToString & GetSessionVars() { return sessionVars; }
+
+    virtual PBoolean TraverseAudio();
+    virtual PBoolean TraverseBreak();
+    virtual PBoolean TraverseValue();
+    virtual PBoolean TraverseSayAs();
+    virtual PBoolean TraverseGoto();
+    virtual PBoolean TraverseGrammar();
+    virtual PBoolean TraverseRecord();
+    virtual PBoolean TraverseIf();
+    virtual PBoolean TraverseExit();
+    virtual PBoolean TraverseVar();
+    virtual PBoolean TraverseSubmit();
+    virtual PBoolean TraverseMenu();
+    virtual PBoolean TraverseChoice();
+    virtual PBoolean TraverseProperty();
+    virtual PBoolean TraverseDisconnect();
+    virtual PBoolean TraverseForm();
+    virtual PBoolean TraversePrompt();
+    virtual PBoolean TraverseField();
+    virtual PBoolean TraversedField();
+    virtual PBoolean TraversedTransfer();
 
   protected:
     void Initialise();
 
-    void AllowClearCall();
-    void ProcessUserInput();
-    void ProcessNode();
-    void ProcessGrammar();
-
-    PBoolean TraverseAudio();
-    PBoolean TraverseGoto();
-    PBoolean TraverseGrammar();
-    PBoolean TraverseRecord();
-
-    PBoolean TraverseIf();
-    PBoolean TraverseExit();
-    PBoolean TraverseVar();
-    PBoolean TraverseSubmit();
-    PBoolean TraverseMenu();
-    PBoolean TraverseChoice(const PString & grammarResult);
-    PBoolean TraverseProperty();
+    virtual void ProcessUserInput();
+    virtual void ProcessGrammar();
+    virtual void ProcessNode();
+    virtual bool NextNode();
 
     void SayAs(const PString & className, const PString & text);
     void SayAs(const PString & className, const PString & text, const PString & voice);
@@ -281,11 +284,10 @@ class PVXMLSession : public PIndirectChannel, public PVXMLChannelInterface
     PXML xmlFile;
 
     PVXMLGrammar * activeGrammar;
-    PBoolean listening;                 // PTrue if waiting for recognition events
+    PBoolean listening;                 // true if waiting for recognition events
     int timeout;                    // timeout in msecs for the current recognition
 
-    PStringToString sessionVars;
-    PStringToString documentVars;
+    PStringToString m_variables;
 
     PMutex userInputMutex;
     std::queue<char> userInputQueue;
@@ -302,8 +304,7 @@ class PVXMLSession : public PIndirectChannel, public PVXMLChannelInterface
     PBoolean emptyAction;
 
     PThread * vxmlThread;
-    PBoolean threadRunning;
-    PBoolean forceEnd;
+    bool m_abortVXML;
 
     PString mediaFormat;
     PVXMLChannel * & vxmlChannel;
@@ -314,10 +315,9 @@ class PVXMLSession : public PIndirectChannel, public PVXMLChannelInterface
     PXMLElement * currentForm;
     PXMLElement * currentField;
     PXMLObject  * currentNode;
+    bool          m_speakNodeData;
 
   private:
-    void      ExecuteDialog();
-
     PString       callingCallToken;
     PSyncPoint    answerSync;
     PString       grammarResult;
@@ -341,7 +341,7 @@ class PVXMLRecordable : public PObject
 
     virtual void OnStart() { }
 
-    virtual PBoolean OnFrame(PBoolean /*isSilence*/) { return PFalse; }
+    virtual PBoolean OnFrame(PBoolean /*isSilence*/) { return false; }
 
     virtual void OnStop() { }
 
@@ -372,7 +372,7 @@ class PVXMLPlayable : public PObject
   PCLASSINFO(PVXMLPlayable, PObject);
   public:
     PVXMLPlayable()
-    { repeat = 1; delay = 0; sampleFrequency = 8000; autoDelete = PFalse; delayDone = PFalse; }
+    { repeat = 1; delay = 0; sampleFrequency = 8000; autoDelete = false; delayDone = false; }
 
     virtual PBoolean Open(PVXMLChannel & /*chan*/, PINDEX delay, PINDEX repeat, PBoolean autoDelete);
 
@@ -405,7 +405,7 @@ class PVXMLPlayable : public PObject
     virtual PBoolean ReadFrame(PVXMLChannel & channel, void * buf, PINDEX len);
 
     virtual PBoolean Rewind(PChannel *) 
-    { return PFalse; }
+    { return false; }
 
     friend class PVXMLChannel;
 
@@ -548,7 +548,7 @@ class PVXMLChannel : public PDelayChannel
     virtual PBoolean Write(const void * buf, PINDEX len);
 
     // new functions
-    virtual PWAVFile * CreateWAVFile(const PFilePath & fn, PBoolean recording = PFalse);
+    virtual PWAVFile * CreateWAVFile(const PFilePath & fn, PBoolean recording = false);
 
     const PString & GetMediaFormat() const { return mediaFormat; }
     PBoolean IsMediaPCM() const { return mediaFormat == "PCM-16"; }
@@ -571,27 +571,26 @@ class PVXMLChannel : public PDelayChannel
 
     virtual PBoolean QueueResource(const PURL & url, PINDEX repeat= 1, PINDEX delay = 0);
 
-    virtual PBoolean QueuePlayable(const PString & type, const PString & str, PINDEX repeat = 1, PINDEX delay = 0, PBoolean autoDelete = PFalse);
+    virtual PBoolean QueuePlayable(const PString & type, const PString & str, PINDEX repeat = 1, PINDEX delay = 0, PBoolean autoDelete = false);
     virtual PBoolean QueuePlayable(PVXMLPlayable * newItem);
     virtual PBoolean QueueData(const PBYTEArray & data, PINDEX repeat = 1, PINDEX delay = 0);
 
-    virtual PBoolean QueueFile(const PString & fn, PINDEX repeat = 1, PINDEX delay = 0, PBoolean autoDelete = PFalse)
+    virtual PBoolean QueueFile(const PString & fn, PINDEX repeat = 1, PINDEX delay = 0, PBoolean autoDelete = false)
     { return QueuePlayable("File", fn, repeat, delay, autoDelete); }
 
     virtual PBoolean QueueCommand(const PString & cmd, PINDEX repeat = 1, PINDEX delay = 0)
-    { return QueuePlayable("Command", cmd, repeat, delay, PTrue); }
+    { return QueuePlayable("Command", cmd, repeat, delay, true); }
 
     virtual void FlushQueue();
-    virtual PBoolean IsPlaying() const   { return (playQueue.GetSize() > 0) || playing ; }
+    virtual PBoolean IsPlaying() const { return currentPlayItem != NULL || playQueue.GetSize() > 0; }
 
     void SetPause(PBoolean pause) { paused = pause; }
 
     void SetName(const PString & name) { channelName = name; }
 
-    unsigned GetSampleFrequency() const
-    { return sampleFrequency; }
+    unsigned GetSampleFrequency() const { return sampleFrequency; }
 
-	  void SetSilentCount(int v) { silentCount = v; }
+    void SetSilence(unsigned msecs);
 
   protected:
     PVXMLChannelInterface * vxmlInterface;
@@ -611,15 +610,13 @@ class PVXMLChannel : public PDelayChannel
     unsigned silenceRun;
 
     // Outgoing audio variables
-    PBoolean playing;
     PMutex queueMutex;
     PVXMLQueue playQueue;
     PVXMLPlayable * currentPlayItem;
 
     PBoolean paused;
-    int silentCount;
+    PTimer   m_silenceTimer;
     int totalData;
-    PTimer delayTimer;
 
     // "channelname" (which is the name of the <record> tag) so
     // results can be saved in vxml session variable
@@ -629,6 +626,17 @@ class PVXMLChannel : public PDelayChannel
 
 //////////////////////////////////////////////////////////////////
 
+class PVXMLNodeHandler : public PObject
+{
+    PCLASSINFO(PVXMLNodeHandler, PObject);
+  public:
+    virtual bool Start(PVXMLSession & /*session*/, const PXMLElement & /*node*/) const { return false; }
+    virtual bool Finish(PVXMLSession & /*session*/, const PXMLElement & /*node*/) const { return false; }
+};
+
+
+typedef PFactory<PVXMLNodeHandler, PCaselessString> PVXMLNodeFactory;
+
 
 #endif // P_VXML
 
