@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23931 $
+ * $Revision: 25231 $
  * $Author: rjongbloed $
- * $Date: 2010-01-13 06:19:08 -0600 (Wed, 13 Jan 2010) $
+ * $Date: 2011-03-02 08:41:46 -0600 (Wed, 02 Mar 2011) $
  */
 
 #ifndef PTLIB_IPSOCKET_H
@@ -157,7 +157,7 @@ class PIPSocket : public PSocket
         bool operator==(int i) const      { return  operator==((DWORD)i); }
         bool operator!=(int i) const      { return !operator==((DWORD)i); }
 
-        /// Compare two addresses for equivalence. This will return PTrue
+        /// Compare two addresses for equivalence. This will return true
         /// if the two addresses are equivalent even if they are IPV6 and IPV4.
 #if P_HAS_IPV6
         bool operator*=(const Address & addr) const;
@@ -170,7 +170,7 @@ class PIPSocket : public PSocket
           bool bracketIPv6 = false ///< An IPv6 address is enclosed in []'s
         ) const;
 
-        /// Convert string to IP address. Returns PTrue if was a valid address.
+        /// Convert string to IP address. Returns true if was a valid address.
         PBoolean FromString(
           const PString & str
         );
@@ -239,6 +239,9 @@ class PIPSocket : public PSocket
 #if P_HAS_IPV6
         /// Check for v4 mapped i nv6 address ::ffff:a.b.c.d.
         PBoolean IsV4Mapped() const;
+
+		/// Check for link-local address fe80::/10
+        PBoolean IsLinkLocal() const;
 #endif
 
         static const Address & GetLoopback(int version = 4);
@@ -309,7 +312,7 @@ class PIPSocket : public PSocket
        PIPSocket::SetPort() function.
 
        @return
-       PTrue if the channel was successfully connected to the remote host.
+       true if the channel was successfully connected to the remote host.
      */
     virtual PBoolean Connect(
       const PString & address   ///< Address of remote machine to connect to.
@@ -344,7 +347,7 @@ class PIPSocket : public PSocket
        For the UDP protocol, the \p queueSize parameter is ignored.
 
        @return
-       PTrue if the channel was successfully opened.
+       true if the channel was successfully opened.
      */
     virtual PBoolean Listen(
       unsigned queueSize = 5,  ///< Number of pending accepts that may be queued.
@@ -380,15 +383,15 @@ class PIPSocket : public PSocket
        specified, for the host this process is running on.
 
        @return
-       PTrue if the IP number was returned.
+       true if the IP number was returned.
      */
     static PBoolean GetHostAddress(
       Address & addr    ///< Variable to receive hosts IP address.
     );
     static PBoolean GetHostAddress(
       const PString & hostname,
-      /* Name of host to get address for. This may be either a domain name or
-         an IP number in "dot" format.
+      /**< Name of host to get address for. This may be either a domain name or
+           an IP number in "dot" format.
        */
       Address & addr    ///< Variable to receive hosts IP address.
     );
@@ -418,7 +421,7 @@ class PIPSocket : public PSocket
        the special number 127.0.0.1 for the loopback device.
 
        @return
-       PTrue if the host is the local machine.
+       true if the host is the local machine.
      */
     static PBoolean IsLocalHost(
       /**Name of host to get address for. This may be either a domain name or
@@ -430,7 +433,7 @@ class PIPSocket : public PSocket
     /**Get the Internet Protocol address and port for the local host.
 
        @return
-       PFalse (or empty string) if the IP number was not available.
+       false (or empty string) if the IP number was not available.
      */
     virtual PString GetLocalAddress();
     virtual PBoolean GetLocalAddress(
@@ -448,7 +451,7 @@ class PIPSocket : public PSocket
        socket is connected to.
 
        @return
-       PFalse (or empty string) if the IP number was not available.
+       false (or empty string) if the IP number was not available.
      */
     virtual PString GetPeerAddress();
     virtual PBoolean GetPeerAddress(
@@ -489,11 +492,11 @@ class PIPSocket : public PSocket
        Note that the driver does not need to be open for this function to work.
 
        @return
-       PTrue if there was a gateway.
+       true if there was a gateway.
      */
     static PBoolean GetGatewayAddress(
       Address & addr,     ///< Variable to receive the IP address.
-	  int version = 4
+	  int version = 4     ///< IP version number
     );
 
     /**Get the name for the interface that is being used as the gateway,
@@ -591,7 +594,7 @@ class PIPSocket : public PSocket
     /**Get the systems route table.
 
        @return
-       PTrue if the route table is returned, PFalse if an error occurs.
+       true if the route table is returned, false if an error occurs.
      */
     static PBoolean GetRouteTable(
       RouteTable & table      ///< Route table
@@ -632,9 +635,6 @@ class PIPSocket : public PSocket
           const Address & addr,
           const Address & mask,
           const PString & macAddr
-#if P_HAS_IPV6
-          , const PString & ip6Addr = PString::Empty()
-#endif
         );
 
         /// Print to specified stream.
@@ -648,16 +648,6 @@ class PIPSocket : public PSocket
         /// Get the address associated with the interface.
         Address GetAddress() const { return ipAddr; }
 
-        PBoolean HasIP6Address() const
-#if ! P_HAS_IPV6
-        { return PFalse;}
-#else
-        { return !ip6Addr.IsEmpty();}
-
-        /// Get the address associated with the interface.
-        Address GetIP6Address() const { return ip6Addr; }
-#endif
-
         /// Get the net mask associated with the interface.
         Address GetNetMask() const { return netMask; }
 
@@ -669,25 +659,24 @@ class PIPSocket : public PSocket
         Address ipAddr;
         Address netMask;
         PString macAddr;
-#if P_HAS_IPV6
-        PString ip6Addr;
-#endif
+
+      friend class PIPSocket;
     };
 
     PARRAY(InterfaceTable, InterfaceEntry);
 
     /**Get a list of all interfaces.
        @return
-       PTrue if the interface table is returned, PFalse if an error occurs.
+       true if the interface table is returned, false if an error occurs.
      */
     static PBoolean GetInterfaceTable(
       InterfaceTable & table,      ///< interface table
-      PBoolean includeDown = PFalse     ///< Include interfaces that are down
+      PBoolean includeDown = false     ///< Include interfaces that are down
     );
 
     /**Get the address of an interface that corresponds to a real network.
        @return
-       PFalse if only loopback interfaces could be found, else PTrue.
+       false if only loopback interfaces could be found, else true.
      */
     static PBoolean GetNetworkInterface(PIPSocket::Address & addr);
 
@@ -695,10 +684,10 @@ class PIPSocket : public PSocket
 
     /**Set flag to capture destination address for incoming packets.
 
-       @return PTrue if host is able to capture incoming address, else PFalse.
+       @return true if host is able to capture incoming address, else false.
       */
     PBoolean SetCaptureReceiveToAddress()
-    { if (!SetOption(IP_PKTINFO, 1, SOL_IP)) return PFalse; catchReceiveToAddr = PTrue; return PTrue; }
+    { if (!SetOption(IP_PKTINFO, 1, SOL_IP)) return false; catchReceiveToAddr = true; return true; }
 
     /**Return the interface address of the last incoming packet.
       */
@@ -715,10 +704,10 @@ class PIPSocket : public PSocket
 
     /**Set flag to capture interface address for incoming packets
 
-       @return PTrue if host is able to capture incoming address, else PFalse
+       @return true if host is able to capture incoming address, else false
       */
     PBoolean SetCaptureReceiveToAddress()
-    { return PFalse; }
+    { return false; }
 
     /**Return the interface address of the last incoming packet.
      */
@@ -765,6 +754,13 @@ class PIPSocketAddressAndPort
       WORD port
     ) { m_port = port; }
 
+    bool IsValid() const { return m_address.IsValid() && m_port != 0; }
+
+    friend ostream & operator<<(ostream & strm, const PIPSocketAddressAndPort & ap)
+    {
+      return strm << ap.m_address << ap.m_separator << ap.m_port;
+    }
+
   protected:
     PIPSocket::Address m_address;
     WORD               m_port;
