@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 24177 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
+ * $Date: 2010-04-05 06:52:04 -0500 (Mon, 05 Apr 2010) $
  */
 
 #ifndef PTLIB_LISTS_H
@@ -48,6 +48,8 @@ struct PListElement
     PListElement * prev;
     PListElement * next;
     PObject * data;
+
+    PDECLARE_POOL_ALLOCATOR();
 };
 
 struct PListInfo
@@ -55,10 +57,12 @@ struct PListInfo
     PListInfo() { head = tail = NULL; }
     PListElement * head;
     PListElement * tail;
+
+    PDECLARE_POOL_ALLOCATOR();
 };
 
 /**This class is a collection of objects which are descendents of the
-   #PObject# class. It is implemeted as a doubly linked list.
+   <code>PObject</code> class. It is implemeted as a doubly linked list.
 
    The implementation of a list allows very fast inserting and deleting of
    objects in the collection, but has severe penalties for random access. All
@@ -69,13 +73,13 @@ struct PListInfo
    access via ordinal index is made sequentially there is little overhead.
 
    The PAbstractList class would very rarely be descended from directly by
-   the user. The #PDECLARE_LIST# and #PLIST# macros would normally
+   the user. The <code>PDECLARE_LIST</code> and <code>PLIST</code> macros would normally
    be used to create descendent classes. They will instantiate the template
-   based on #PList# or directly declare and define the class (using
+   based on <code>PList</code> or directly declare and define the class (using
    inline functions) if templates are not being used.
 
-   The #PList# class or #PDECLARE_LIST# macro will define the
-   correctly typed operators for subscript access (#operator[]#).
+   The <code>PList</code> class or <code>PDECLARE_LIST</code> macro will define the
+   correctly typed operators for subscript access (operator[]).
  */
 class PAbstractList : public PCollection
 {
@@ -95,32 +99,33 @@ class PAbstractList : public PCollection
   // Overrides from class PObject
     /**Get the relative rank of the two lists. The following algorithm is
        employed for the comparison:
-\begin{description}
-       \item[#EqualTo#] if the two lists are identical in length
+
+       \arg \c EqualTo if the two lists are identical in length
        and each objects values, not pointer, are equal.
 
-       \item[#LessThan#] if the instances object value at an
+       \arg \c LessThan if the instances object value at an
        ordinal position is less than the corresponding objects value in the
-       #obj# parameters list.
+       <code>obj</code> parameters list.
                           
        This is also returned if all objects are equal and the instances list
-       length is less than the #obj# parameters list length.
+       length is less than the <code>obj</code> parameters list length.
 
-       \item[#GreaterThan#] if the instances object value at an
+       \arg \c GreaterThan if the instances object value at an
        ordinal position is greater than the corresponding objects value in the
-       #obj# parameters list.
+       <code>obj</code> parameters list.
                           
        This is also returned if all objects are equal and the instances list
-       length is greater than the #obj# parameters list length.
-\end{description}
+       length is greater than the <code>obj</code> parameters list length.
 
        @return
-       comparison of the two objects, #EqualTo# for same,
-       #LessThan# for #obj# logically less than the
-       object and #GreaterThan# for #obj# logically
+       comparison of the two objects, <code>EqualTo</code> for same,
+       <code>LessThan</code> for <code>obj</code> logically less than the
+       object and <code>GreaterThan</code> for <code>obj</code> logically
        greater than the object.
      */
-    virtual Comparison Compare(const PObject & obj) const;
+    virtual Comparison Compare(
+      const PObject & obj   ///< Object being compared to
+    ) const;
 
   /**@name Overrides from class PContainer */
   //@{
@@ -129,7 +134,7 @@ class PAbstractList : public PCollection
        set in any other way.
 
        @return
-       Always PTrue.
+       Always true.
      */
     virtual PBoolean SetSize(
       PINDEX newSize  ///< New size for the list, this is ignored.
@@ -150,11 +155,11 @@ class PAbstractList : public PCollection
 
     /**Insert a new object immediately before the specified object. If the
        object to insert before is not in the collection then the equivalent of
-       the #Append()# function is performed.
+       the <code>Append()</code> function is performed.
        
        Note that the object values are compared for the search of the
-       #before# parameter, not the pointers. So the objects in the
-       collection must correctly implement the #PObject::Compare()#
+       <code>before</code> parameter, not the pointers. So the objects in the
+       collection must correctly implement the <code>PObject::Compare()</code>
        function.
 
        @return
@@ -167,7 +172,7 @@ class PAbstractList : public PCollection
 
     /**Insert a new object at the specified ordinal index. If the index is
        greater than the number of objects in the collection then the
-       equivalent of the #Append()# function is performed.
+       equivalent of the <code>Append()</code> function is performed.
 
        @return
        index of the newly inserted object.
@@ -181,7 +186,7 @@ class PAbstractList : public PCollection
        is set then the object is also deleted.
 
        @return
-       PTrue if the object was in the collection.
+       true if the object was in the collection.
      */
     virtual PBoolean Remove(
       const PObject * obj   ///< Existing object to remove from the collection.
@@ -203,13 +208,13 @@ class PAbstractList : public PCollection
     /**Set the object at the specified ordinal position to the new value. This
        will overwrite the existing entry. 
        This method will NOT delete the old object independently of the 
-       AllowDeleteObjects option. Use #ReplaceAt()# instead.
+       AllowDeleteObjects option. Use <code>ReplaceAt()</code> instead.
 
        Note if the index is beyond the size of the collection then the
        function will assert.
 
        @return
-       PTrue if the object was successfully added.
+       true if the object was successfully added.
      */
     virtual PBoolean SetAt(
       PINDEX index,   ///< Index position in collection to set.
@@ -224,7 +229,7 @@ class PAbstractList : public PCollection
        function will assert.
        
        @return
-       PTrue if the object was successfully replaced.
+       true if the object was successfully replaced.
      */   
     virtual PBoolean ReplaceAt(
       PINDEX index,   ///< Index position in collection to set.
@@ -258,7 +263,7 @@ class PAbstractList : public PCollection
 
     /**Search the collection for the specified value of the object. The object
        values are compared, not the pointers.  So the objects in the
-       collection must correctly implement the #PObject::Compare()#
+       collection must correctly implement the <code>PObject::Compare()</code>
        function. A simple linear search from "head" of the list is performed.
 
        @return
@@ -292,7 +297,7 @@ class PAbstractList : public PCollection
        point for a sequential move to the required index.
 
        @return
-       PTrue if the index could be set as the current element.
+       true if the index could be set as the current element.
      */
     PBoolean SetCurrent(
       PINDEX index,           ///< Ordinal index of the list element to set as current.
@@ -310,7 +315,7 @@ class PAbstractList : public PCollection
 /**This template class maps the PAbstractList to a specific object type. The
    functions in this class primarily do all the appropriate casting of types.
 
-   Note that if templates are not used the #PDECLARE_LIST# macro will
+   Note that if templates are not used the <code>PDECLARE_LIST</code> macro will
    simulate the template instantiation.
  */
 template <class T> class PList : public PAbstractList
@@ -410,10 +415,11 @@ template <class T> class PList : public PAbstractList
        element, and the head and tail of the list, will always be fast.
 
        @return
-       reference to the object at #index# position.
+       reference to the object at <code>index</code> position.
      */
-    T & operator[](PINDEX index) const
-      { return (T &)GetReferenceAt(index); }
+    T & operator[](
+      PINDEX index  ///< Index for entry
+    ) const { return (T &)GetReferenceAt(index); }
   //@}
 
   protected:
@@ -424,27 +430,27 @@ template <class T> class PList : public PAbstractList
 
 /**Declare a list class.
    This macro is used to declare a descendent of PAbstractList class,
-   customised for a particular object type {\bf T}. This macro closes the
+   customised for a particular object type <b>T</b>. This macro closes the
    class declaration off so no additional members can be added.
 
    If the compilation is using templates then this macro produces a typedef
-   of the #PList# template class.
+   of the <code>PList</code> template class.
 
-   See the #PList# class and #PDECLARE_LIST# macro for more
+   See the <code>PList</code> class and <code>PDECLARE_LIST</code> macro for more
    information.
  */
 #define PLIST(cls, T) typedef PList<T> cls
 
 /**Begin declaration of list class.
    This macro is used to declare a descendent of PAbstractList class,
-   customised for a particular object type {\bf T}.
+   customised for a particular object type <b>T</b>.
 
    If the compilation is using templates then this macro produces a descendent
-   of the #PList# template class. If templates are not being used then the
+   of the <code>PList</code> template class. If templates are not being used then the
    macro defines a set of inline functions to do all casting of types. The
    resultant classes have an identical set of functions in either case.
 
-   See the #PList# and #PAbstractList# classes for more information.
+   See the <code>PList</code> and <code>PAbstractList</code> classes for more information.
  */
 #define PDECLARE_LIST(cls, T) \
   PLIST(cls##_PTemplate, T); \
@@ -464,11 +470,11 @@ template <class T> class PList : public PAbstractList
    queue. The functions in this class primarily do all the appropriate casting
    of types.
 
-   By default, objects placed into the set will {\bf not} be deleted when
+   By default, objects placed into the set will <b>T</b> be deleted when
    removed or when all references to the set are destroyed. This is different
    from the default on most collection classes.
 
-   Note that if templates are not used the #PDECLARE_QUEUE# macro will
+   Note that if templates are not used the <code>PDECLARE_QUEUE</code> macro will
    simulate the template instantiation.
  */
 template <class T> class PQueue : public PAbstractList
@@ -480,7 +486,7 @@ template <class T> class PQueue : public PAbstractList
   //@{
     /**Create a new, empty, queue.
 
-       Note that by default, objects placed into the queue will {\bf not} be
+       Note that by default, objects placed into the queue will <b>not</b> be
        deleted when removed or when all references to the queue are destroyed.
        This is different from the default on most collection classes.
      */
@@ -523,14 +529,14 @@ template <class T> class PQueue : public PAbstractList
 
 /**Declare a queue class.
    This macro is used to declare a descendent of PAbstractList class,
-   customised for a particular object type {\bf T}, and adds functionality
+   customised for a particular object type <b>T</b>, and adds functionality
    that allows the list to be used as a first in first out queue. This macro
    closes the class declaration off so no additional members can be added.
 
    If the compilation is using templates then this macro produces a typedef
-   of the #PQueue# template class.
+   of the <code>PQueue</code> template class.
 
-   See the #PList# class and #PDECLARE_QUEUE# macro for more
+   See the <code>PList</code> class and <code>PDECLARE_QUEUE</code> macro for more
    information.
  */
 #define PQUEUE(cls, T) typedef PQueue<T> cls
@@ -538,15 +544,15 @@ template <class T> class PQueue : public PAbstractList
 
 /**Begin declataion of a queue class.
    This macro is used to declare a descendent of PAbstractList class,
-   customised for a particular object type {\bf T}, and adds functionality
+   customised for a particular object type <b>T</b>, and adds functionality
    that allows the list to be used as a first in first out queue.
 
    If the compilation is using templates then this macro produces a descendent
-   of the #PQueue# template class. If templates are not being used then
+   of the <code>PQueue</code> template class. If templates are not being used then
    the macro defines a set of inline functions to do all casting of types. The
    resultant classes have an identical set of functions in either case.
 
-   See the #PQueue# and #PAbstractList# classes for more information.
+   See the <code>PQueue</code> and <code>PAbstractList</code> classes for more information.
  */
 #define PDECLARE_QUEUE(cls, T) \
   PQUEUE(cls##_PTemplate, T); \
@@ -566,11 +572,11 @@ template <class T> class PQueue : public PAbstractList
    stack. The functions in this class primarily do all the appropriate casting
    of types.
 
-   By default, objects placed into the set will {\bf not} be deleted when
+   By default, objects placed into the set will <b>not</b> be deleted when
    removed or when all references to the set are destroyed. This is different
    from the default on most collection classes.
 
-   Note that if templates are not used the #PDECLARE_STACK# macro will
+   Note that if templates are not used the <code>PDECLARE_STACK</code> macro will
    simulate the template instantiation.
  */
 template <class T> class PStack : public PAbstractList
@@ -582,7 +588,7 @@ template <class T> class PStack : public PAbstractList
   //@{
     /**Create a new, empty, stack.
 
-       Note that by default, objects placed into the stack will {\bf not} be
+       Note that by default, objects placed into the stack will <b>not</b> be
        deleted when removed or when all references to the stack are destroyed.
        This is different from the default on most collection classes.
      */
@@ -602,7 +608,7 @@ template <class T> class PStack : public PAbstractList
   /**@name New functions for class */
   //@{
     /**Add an object to the stack. This object will be on "top" of the stack
-       and will be the object returned by the #Pop()#
+       and will be the object returned by the <code>Pop()</code>
        function.
      */
     virtual void Push(
@@ -636,14 +642,14 @@ template <class T> class PStack : public PAbstractList
 
 /**Declare a stack class.
    This macro is used to declare a descendent of PAbstractList class,
-   customised for a particular object type {\bf T}, and adds functionality
+   customised for a particular object type <b>T</b>, and adds functionality
    that allows the list to be used as a last in first out stack. This macro
    closes the class declaration off so no additional members can be added.
 
    If the compilation is using templates then this macro produces a typedef
-   of the #PStack# template class.
+   of the <code>PStack</code> template class.
 
-   See the #PStack# class and #PDECLARE_STACK# macro for more
+   See the <code>PStack</code> class and <code>PDECLARE_STACK</code> macro for more
    information.
  */
 #define PSTACK(cls, T) typedef PStack<T> cls
@@ -651,15 +657,15 @@ template <class T> class PStack : public PAbstractList
 
 /**Begin declaration of a stack class.
    This macro is used to declare a descendent of PAbstractList class,
-   customised for a particular object type {\bf T}, and adds functionality
+   customised for a particular object type <b>T</b>, and adds functionality
    that allows the list to be used as a last in first out stack.
 
    If the compilation is using templates then this macro produces a descendent
-   of the #PStack# template class. If templates are not being used then
+   of the <code>PStack</code> template class. If templates are not being used then
    the macro defines a set of inline functions to do all casting of types. The
    resultant classes have an identical set of functions in either case.
 
-   See the #PStack# and #PAbstractList# classes for more information.
+   See the <code>PStack</code> and <code>PAbstractList</code> classes for more information.
  */
 #define PDECLARE_STACK(cls, T) \
   PSTACK(cls##_PTemplate, T); \
@@ -685,6 +691,8 @@ struct PSortedListElement
   PObject            * data;
   PINDEX               subTreeSize;
   enum { Red, Black }  colour;
+
+  PDECLARE_POOL_ALLOCATOR();
 };
 
 struct PSortedListInfo
@@ -701,12 +709,14 @@ struct PSortedListInfo
   PSortedListElement * OrderSelect(PSortedListElement * node, PINDEX index) const;
 
   typedef PSortedListElement Element;
+
+  PDECLARE_POOL_ALLOCATOR();
 };
 
 /**This class is a collection of objects which are descendents of the
-   #PObject# class. It is implemeted as a Red-Black binary tree to
+   <code>PObject</code> class. It is implemeted as a Red-Black binary tree to
    maintain the objects in rank order. Note that this requires that the
-   #PObject::Compare()# function be fully implemented oin objects
+   <code>PObject::Compare()</code> function be fully implemented oin objects
    contained in the collection.
 
    The implementation of a sorted list allows fast inserting and deleting as
@@ -720,14 +730,14 @@ struct PSortedListInfo
    other access incurs a minimum overhead, but not insignificant.
 
    The PAbstractSortedList class would very rarely be descended from directly
-   by the user. The #PDECLARE_LIST# and #PLIST# macros would normally
+   by the user. The <code>PDECLARE_LIST</code> and <code>PLIST</code> macros would normally
    be used to create descendent classes. They will instantiate the template
-   based on #PSortedList# or directly declare and define the class (using
+   based on <code>PSortedList</code> or directly declare and define the class (using
    inline functions) if templates are not being used.
 
-   The #PSortedList# class or #PDECLARE_SORTED_LIST# macro will
+   The <code>PSortedList</code> class or <code>PDECLARE_SORTED_LIST</code> macro will
    define the correctly typed operators for subscript access
-   (#operator[]#).
+   (operator[]).
  */
 class PAbstractSortedList : public PCollection
 {
@@ -748,29 +758,28 @@ class PAbstractSortedList : public PCollection
   //@{
     /**Get the relative rank of the two lists. The following algorithm is
        employed for the comparison:
-\begin{descriptions}
-       \item[#EqualTo#] if the two lists are identical in length
+
+       \arg \c EqualTo if the two lists are identical in length
        and each objects values, not pointer, are equal.
 
-       \item[#LessThan#] if the instances object value at an
+       \arg \c LessThan if the instances object value at an
        ordinal position is less than the corresponding objects value in the
-       #obj# parameters list.
+       <code>obj</code> parameters list.
                           
        This is also returned if all objects are equal and the instances list
-       length is less than the #obj# parameters list length.
+       length is less than the <code>obj</code> parameters list length.
 
-       \item[#GreaterThan#] if the instances object value at an
+       \arg \c GreaterThan if the instances object value at an
        ordinal position is greater than the corresponding objects value in the
-       #obj# parameters list.
+       <code>obj</code> parameters list.
                           
        This is also returned if all objects are equal and the instances list
-       length is greater than the #obj# parameters list length.
-\end{descriptions}
+       length is greater than the <code>obj</code> parameters list length.
 
        @return
-       comparison of the two objects, #EqualTo# for same,
-       #LessThan# for #obj# logically less than the
-       object and #GreaterThan# for #obj# logically
+       comparison of the two objects, <code>EqualTo</code> for same,
+       <code>LessThan</code> for <code>obj</code> logically less than the
+       object and <code>GreaterThan</code> for <code>obj</code> logically
        greater than the object.
      */
     virtual Comparison Compare(const PObject & obj) const;
@@ -783,7 +792,7 @@ class PAbstractSortedList : public PCollection
        set in any other way.
 
        @return
-       Always PTrue.
+       Always true.
      */
     virtual PBoolean SetSize(
       PINDEX newSize  // New size for the sorted list, this is ignored.
@@ -805,7 +814,7 @@ class PAbstractSortedList : public PCollection
     /**Add a new object to the collection.
     
        The object is always placed in the correct ordinal position in the list.
-       It is not placed at the specified position. The #before#
+       It is not placed at the specified position. The <code>before</code>
        parameter is ignored.
 
        @return
@@ -819,7 +828,7 @@ class PAbstractSortedList : public PCollection
     /**Add a new object to the collection.
     
        The object is always placed in the correct ordinal position in the list.
-       It is not placed at the specified position. The #index#
+       It is not placed at the specified position. The <code>index</code>
        parameter is ignored.
 
        @return
@@ -838,7 +847,7 @@ class PAbstractSortedList : public PCollection
        same instance of the object that is in the collection.
 
        @return
-       PTrue if the object was in the collection.
+       true if the object was in the collection.
      */
     virtual PBoolean Remove(
       const PObject * obj   // Existing object to remove from the collection.
@@ -858,18 +867,18 @@ class PAbstractSortedList : public PCollection
     );
 
     /**Remove all of the elements in the collection. This operates by
-       continually calling #RemoveAt()# until there are no objects left.
+       continually calling <code>RemoveAt()</code> until there are no objects left.
 
        The objects are removed from the last, at index
-       #(GetSize()-1)# toward the first at index zero.
+       <code>(GetSize()-1)</code> toward the first at index zero.
      */
     virtual void RemoveAll();
 
-    /**This method simply returns PFalse as the list order is mantained by the 
-       class. Kept to mimic #PAbstractList# interface.
+    /**This method simply returns false as the list order is mantained by the 
+       class. Kept to mimic <code>PAbstractList</code> interface.
        
        @return
-       PFalse allways
+       false allways
      */
     virtual PBoolean SetAt(
       PINDEX index,   // Index position in collection to set.
@@ -907,7 +916,7 @@ class PAbstractSortedList : public PCollection
 
     /**Search the collection for the specified value of the object. The object
        values are compared, not the pointers.  So the objects in the
-       collection must correctly implement the #PObject::Compare()#
+       collection must correctly implement the <code>PObject::Compare()</code>
        function. A binary search is employed to locate the entry.
 
        @return
@@ -939,7 +948,7 @@ class PAbstractSortedList : public PCollection
    The functions in this class primarily do all the appropriate casting of
    types.
 
-   Note that if templates are not used the #PDECLARE_SORTED_LIST# macro
+   Note that if templates are not used the <code>PDECLARE_SORTED_LIST</code> macro
    will simulate the template instantiation.
  */
 template <class T> class PSortedList : public PAbstractSortedList
@@ -977,10 +986,11 @@ template <class T> class PSortedList : public PAbstractSortedList
        access will be fast.
 
        @return
-       reference to the object at #index# position.
+       reference to the object at <code>index</code> position.
      */
-    T & operator[](PINDEX index) const
-      { return *(T *)GetAt(index); }
+    T & operator[](
+      PINDEX index  ///< Index for entry
+    ) const { return *(T *)GetAt(index); }
   //@}
 
   protected:
@@ -991,13 +1001,13 @@ template <class T> class PSortedList : public PAbstractSortedList
 
 /**Declare a sorted list class.
    This macro is used to declare a descendent of PAbstractSortedList class,
-   customised for a particular object type {\bf T}. This macro closes the
+   customised for a particular object type <b>T</b>. This macro closes the
    class declaration off so no additional members can be added.
 
    If the compilation is using templates then this macro produces a typedef
-   of the #PSortedList# template class.
+   of the <code>PSortedList</code> template class.
 
-   See the #PSortedList# class and #PDECLARE_SORTED_LIST# macro for
+   See the <code>PSortedList</code> class and <code>PDECLARE_SORTED_LIST</code> macro for
    more information.
  */
 #define PSORTED_LIST(cls, T) typedef PSortedList<T> cls
@@ -1005,14 +1015,14 @@ template <class T> class PSortedList : public PAbstractSortedList
 
 /**Begin declaration of a sorted list class.
    This macro is used to declare a descendent of PAbstractSortedList class,
-   customised for a particular object type {\bf T}.
+   customised for a particular object type <b>T</b>.
 
    If the compilation is using templates then this macro produces a descendent
-   of the #PSortedList# template class. If templates are not being used
+   of the <code>PSortedList</code> template class. If templates are not being used
    then the macro defines a set of inline functions to do all casting of types.
    The resultant classes have an identical set of functions in either case.
 
-   See the #PSortedList# and #PAbstractSortedList# classes for more
+   See the <code>PSortedList</code> and <code>PAbstractSortedList</code> classes for more
    information.
  */
 #define PDECLARE_SORTED_LIST(cls, T) \
