@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23340 $
+ * $Revision: 24492 $
  * $Author: rjongbloed $
- * $Date: 2009-09-04 22:25:03 -0500 (Fri, 04 Sep 2009) $
+ * $Date: 2010-06-14 07:13:30 -0500 (Mon, 14 Jun 2010) $
  */
 
 #ifndef PTLIB_CONTAIN_H
@@ -44,6 +44,7 @@
 
 #ifdef _MSC_VER
 
+  #pragma warning(disable:4201)  // nonstandard extension: nameless struct/union
   #pragma warning(disable:4201)  // nonstandard extension: nameless struct/union
   #pragma warning(disable:4251)  // disable warning exported structs
   #pragma warning(disable:4511)  // default copy ctor not generated warning
@@ -95,6 +96,10 @@
     #define _WIN32_WINNT WINVER
   #endif
 
+  #if !defined(_WIN32_WCE) && defined(_WIN32_WINNT) && (_WIN32_WINNT == 0x0500) && P_HAS_IPV6 && !defined(NTDDI_VERSION)
+    #define NTDDI_VERSION NTDDI_WIN2KSP1
+  #endif
+
   #ifndef STRICT
     #define STRICT
   #endif
@@ -236,8 +241,6 @@ typedef long PInt32;
 
 #endif // !_WIN32_WCE 
 
-#define _hptr(x) (PINDEX)(x)
-
 
 class PWin32Overlapped : public OVERLAPPED
 {
