@@ -7,16 +7,14 @@
  *
  * Contributor(s): Snark at GnomeMeeting
  *
- * $Revision: 21788 $
+ * $Revision: 26272 $
  * $Author: rjongbloed $
- * $Date: 2008-12-11 23:42:13 -0600 (Thu, 11 Dec 2008) $
+ * $Date: 2011-08-03 19:31:06 -0500 (Wed, 03 Aug 2011) $
  */
 
 #ifndef PTLIB_PLUGINMGR_H
 #define PTLIB_PLUGINMGR_H
 
-#define DEFAULT_PLUGINDIR "/usr/lib/pwlib"
-
 #include <ptlib/plugin.h>
 
 class PPluginSuffix {
@@ -64,6 +62,8 @@ class PPluginManager : public PObject
     // functions to load/unload a dynamic plugin 
     PBoolean LoadPlugin (const PString & fileName);
     void LoadPluginDirectory (const PDirectory & dir);
+
+    void OnShutdown();
   
     // functions to access the plugins' services 
     PStringArray GetPluginTypes() const;
@@ -86,6 +86,11 @@ class PPluginManager : public PObject
     // static functions for accessing global instances of plugin managers
     static PPluginManager & GetPluginManager();
 
+    enum NotificationCode {
+      LoadingPlugIn,
+      UnloadingPlugIn
+    };
+
     /**Add a notifier to the plugin manager.
        The call back function is executed just after loading, or 
        just after unloading, a plugin. 
@@ -105,7 +110,7 @@ class PPluginManager : public PObject
 
     void AddNotifier(
       const PNotifier & filterFunction,
-      PBoolean existing = PFalse
+      PBoolean existing = false
     );
 
     void RemoveNotifier(
@@ -113,18 +118,19 @@ class PPluginManager : public PObject
     );
 
     static PString additionalPluginDirs;
+
   protected:
     void LoadPluginDirectory (const PDirectory & directory, const PStringList & suffixes);
-    void CallNotifier(PDynaLink & dll, INT code);
+    void CallNotifier(PDynaLink & dll, NotificationCode code);
 
-    PMutex pluginsMutex;
-    PArray<PDynaLink> plugins;
+    PMutex            m_pluginsMutex;
+    PArray<PDynaLink> m_plugins;
     
-    PMutex servicesMutex;
-    PArray<PPluginService> services;
+    PMutex                 m_servicesMutex;
+    PArray<PPluginService> m_services;
 
-    PMutex notifiersMutex;
-    PList<PNotifier> notifiers;
+    PMutex           m_notifiersMutex;
+    PList<PNotifier> m_notifiers;
 };
 
 //////////////////////////////////////////////////////
@@ -140,7 +146,7 @@ class PPluginModuleManager : public PObject
     PPluginModuleManager(const char * signatureFunctionName, PPluginManager * pluginMgr = NULL);
 
     PBoolean LoadPlugin(const PString & fileName)
-    { if (pluginMgr == NULL) return PFalse; else return pluginMgr->LoadPlugin(fileName); }
+    { if (pluginMgr == NULL) return false; else return pluginMgr->LoadPlugin(fileName); }
 
     void LoadPluginDirectory(const PDirectory &directory)
     { if (pluginMgr != NULL) pluginMgr->LoadPluginDirectory(directory); }
@@ -164,6 +170,9 @@ class PPluginModuleManager : public PObject
 };
 
 
+PFACTORY_LOAD(PluginLoaderStartup);
+
+
 #endif // PTLIB_PLUGINMGR_H
 
 
