@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22356 $
+ * $Revision: 24524 $
  * $Author: rjongbloed $
- * $Date: 2009-03-31 23:57:19 +0000 (Tue, 31 Mar 2009) $
+ * $Date: 2010-06-21 20:05:24 -0500 (Mon, 21 Jun 2010) $
  */
 
 #ifndef PTLIB_PROCESS_H
@@ -44,11 +44,12 @@
 #include <ptlib/pfactory.h>
 
 #include <queue>
+#include <set>
 
 /**Create a process.
    This macro is used to create the components necessary for a user PWLib
    process. For a PWLib program to work correctly on all platforms the
-   #main()# function must be defined in the same module as the
+   main() function must be defined in the same module as the
    instance of the application.
  */
 #ifdef P_VXWORKS
@@ -89,7 +90,7 @@ extern "C" {\
 /*$MACRO PDECLARE_PROCESS(cls,ancestor,manuf,name,major,minor,status,build)
    This macro is used to declare the components necessary for a user PWLib
    process. This will declare the PProcess descendent class, eg PApplication,
-   and create an instance of the class. See the #PCREATE_PROCESS# macro
+   and create an instance of the class. See the <code>PCREATE_PROCESS</code> macro
    for more details.
  */
 #define PDECLARE_PROCESS(cls,ancestor,manuf,name,major,minor,status,build) \
@@ -103,9 +104,9 @@ extern "C" {\
 
 
 class PTimerList : public PObject
-/* This class defines a list of #PTimer# objects. It is primarily used
+/* This class defines a list of <code>PTimer</code> objects. It is primarily used
    internally by the library and the user should never create an instance of
-   it. The #PProcess# instance for the application maintains an instance
+   it. The <code>PProcess</code> instance for the application maintains an instance
    of all of the timers created so that it may decrements them at regular
    intervals.
  */
@@ -117,7 +118,7 @@ class PTimerList : public PObject
     PTimerList();
 
     /* Decrement all the created timers and dispatch to their callback
-       functions if they have expired. The #PTimer::Tick()# function
+       functions if they have expired. The <code>PTimer::Tick()</code> function
        value is used to determine the time elapsed since the last call to
        Process().
 
@@ -137,44 +138,74 @@ class PTimerList : public PObject
         enum Action {
           Stop,
           Start
-        } action;
+        } m_action;
 
         RequestType(Action act, PTimer * t)
-          : action(act)
-          , timer(t)
-          , id(timer->GetTimerId())
-          , sync(NULL)
+          : m_action(act)
+          , m_timer(t)
+          , m_id(t->GetTimerId())
+          , m_absoluteTime(t->GetAbsoluteTime())
+          , m_serialNumber(t->GetNextSerialNumber())
+          , m_sync(NULL)
         { }
 
-        PTimer * timer;
-        PTimer::IDType id;
-        PSyncPoint * sync;
+        PTimer *                    m_timer;
+        PTimer::IDType              m_id;
+        PInt64                      m_absoluteTime;
+        PAtomicInteger::IntegerType m_serialNumber;
+        PSyncPoint *                m_sync;
     };
 
     void QueueRequest(RequestType::Action action, PTimer * timer, bool isSync = true);
 
+    void ProcessTimerQueue();
+
   private:
+    // queue of timer action requests
+    PMutex m_queueMutex;
+    typedef std::queue<RequestType> RequestQueueType;
+    RequestQueueType m_requestQueue;
+
+    // add an active timer to the lists
+    void AddActiveTimer(const RequestType & request);
+
+    //  counter to keep track of timer IDs
     mutable PAtomicInteger timerId; 
 
-    // map used to store timer information
-    PMutex timerListMutex;
-    struct TimerInfoType {
-      TimerInfoType(PTimer * t) : timer(t)  { removed = false; }
-      PTimer * timer;
-      bool removed;
+    // map used to store active timer information
+    struct ActiveTimerInfo {
+      ActiveTimerInfo(PTimer * t, PAtomicInteger::IntegerType serialNumber) 
+        : m_timer(t), m_serialNumber(serialNumber) { }
+      PTimer * m_timer;
+      PAtomicInteger::IntegerType m_serialNumber;
     };
-    typedef std::map<PTimer::IDType, TimerInfoType> TimerInfoMapType;
-    TimerInfoMapType activeTimers;
-    PThread * timerThread;
+    typedef std::map<PTimer::IDType, ActiveTimerInfo> ActiveTimerInfoMap;
+    ActiveTimerInfoMap m_activeTimers;
 
-    // queue of timer action requests
-    PMutex queueMutex;
-    typedef std::queue<RequestType> RequestQueueType;
-    RequestQueueType requestQueue;
-    RequestQueueType addQueue;
+    // set used to store timer expiry times, in order
+    struct TimerExpiryInfo {
+      TimerExpiryInfo(PTimer::IDType id, PInt64 expireTime, PAtomicInteger::IntegerType serialNumber)
+        : m_timerId(id), m_expireTime(expireTime), m_serialNumber(serialNumber) { }
+      PTimer::IDType m_timerId;
+      PInt64 m_expireTime;
+      PAtomicInteger::IntegerType m_serialNumber;
+    };
+
+	  struct TimerExpiryInfo_compare
+		  : public binary_function<TimerExpiryInfo, TimerExpiryInfo, bool>
+	  {	
+	    bool operator()(const TimerExpiryInfo & _Left, const TimerExpiryInfo & _Right) const
+		  {	return (_Left.m_expireTime < _Right.m_expireTime); }
+	  };
+
+    typedef std::multiset<TimerExpiryInfo, TimerExpiryInfo_compare> TimerExpiryInfoList;
+    TimerExpiryInfoList m_expiryList;
 
     // The last system timer tick value that was used to process timers.
-    PTimeInterval lastSample;
+    PTimeInterval m_lastSample;
+
+    // thread that handles the timer stuff
+    PThread * m_timerThread;
 };
 
 
@@ -185,12 +216,12 @@ class PTimerList : public PObject
    "programme" in the  context of the operating system. Note that there can
    only be one instance of a PProcess class in a given programme.
    
-   The instance of a PProcess or its GUI descendent #PApplication# is
+   The instance of a PProcess or its GUI descendent <code>PApplication</code> is
    usually a static variable created by the application writer. This is the
    initial "anchor" point for all data structures in an application. As the
    application writer never needs to access the standard system
-   #main()# function, it is in the library, the programmes
-   execution begins with the virtual function #PThread::Main()# on a
+   <code>main()</code> function, it is in the library, the programmes
+   execution begins with the virtual function <code>PThread::Main()</code> on a
    process.
  */
 class PProcess : public PThread
@@ -230,7 +261,7 @@ class PProcess : public PThread
        a programme only has access to a single process, its own.
 
        @return
-       #EqualTo# if the two process object have the same name.
+       <code>EqualTo</code> if the two process object have the same name.
      */
     Comparison Compare(
       const PObject & obj   ///< Other process to compare against.
@@ -262,7 +293,7 @@ class PProcess : public PThread
 
   /**@name Process information functions */
   //@{
-    /**Get the current processes object instance. The {\it current process}
+    /**Get the current processes object instance. The <i>current process</i>
        is the one the application is running in.
        
        @return
@@ -294,17 +325,17 @@ class PProcess : public PThread
 
        Default behaviour returns false and the process is killed.
 
-       @Return true if the process is to be allowed to continue, false otherwise.
+       @return true if the process is to be allowed to continue, false otherwise.
       */
     virtual bool OnInterrupt(
       bool terminating ///< true if process terminating.
     );
 
     /**Determine if the current processes object instance has been initialised.
-       If this returns PTrue it is safe to use the PProcess::Current() function.
+       If this returns true it is safe to use the PProcess::Current() function.
        
        @return
-       PTrue if process class has been initialised.
+       true if process class has been initialised.
      */
     static PBoolean IsInitialised();
 
@@ -339,7 +370,7 @@ class PProcess : public PThread
 
     /**Get the name of the manufacturer of the software. This is used in the
        default "About" dialog box and for determining the location of the
-       configuration information as used by the #PConfig# class.
+       configuration information as used by the <code>PConfig</code> class.
 
        The default for this information is the empty string.
     
@@ -350,7 +381,7 @@ class PProcess : public PThread
 
     /**Get the name of the process. This is used in the
        default "About" dialog box and for determining the location of the
-       configuration information as used by the #PConfig# class.
+       configuration information as used by the <code>PConfig</code> class.
 
        The default is the title part of the executable image file.
 
@@ -361,11 +392,11 @@ class PProcess : public PThread
 
     /**Get the version of the software. This is used in the default "About"
        dialog box and for determining the location of the configuration
-       information as used by the #PConfig# class.
+       information as used by the <code>PConfig</code> class.
 
-       If the #full# parameter is PTrue then a version string
+       If the <code>full</code> parameter is true then a version string
        built from the major, minor, status and build veriosn codes is
-       returned. If PFalse then only the major and minor versions are
+       returned. If false then only the major and minor versions are
        returned.
 
        The default for this information is "1.0".
@@ -374,7 +405,7 @@ class PProcess : public PThread
        string for the version eg "1.0b3".
      */
     virtual PString GetVersion(
-      PBoolean full = PTrue ///< PTrue for full version, PFalse for short version.
+      PBoolean full = true ///< true for full version, false for short version.
     ) const;
 
     /**Get the processes executable image file path.
@@ -426,12 +457,12 @@ class PProcess : public PThread
        meaningful for all platforms.
 
        @return
-       PTrue if processes owner changed. The most common reason for failure is
+       true if processes owner changed. The most common reason for failure is
        that the process does not have the privilege to change the effective user.
       */
     PBoolean SetUserName(
       const PString & username, ///< New user name or uid
-      PBoolean permanent = PFalse    ///< Flag for if effective or real user
+      PBoolean permanent = false    ///< Flag for if effective or real user
     );
 
     /**Get the effective group name of the owner of the process, eg "root" etc.
@@ -464,13 +495,13 @@ class PProcess : public PThread
        meaningful for all platforms.
 
        @return
-       PTrue if processes group changed. The most common reason for failure is
+       true if processes group changed. The most common reason for failure is
        that the process does not have the privilege to change the effective
        group.
       */
     PBoolean SetGroupName(
       const PString & groupname, ///< New group name or gid
-      PBoolean permanent = PFalse     ///< Flag for if effective or real group
+      PBoolean permanent = false     ///< Flag for if effective or real group
     );
 
     /**Get the maximum file handle value for the process.
@@ -488,7 +519,7 @@ class PProcess : public PThread
        For some platforms this is meaningless.
 
        @return
-       PTrue if successfully set the maximum file hadles.
+       true if successfully set the maximum file hadles.
       */
     PBoolean SetMaxHandles(
       int newLimit  ///< New limit on file handles
@@ -551,6 +582,17 @@ class PProcess : public PThread
      */
     static PString GetOSVersion();
 
+    /**See if operating system is later than the version specified.
+
+       @return
+       true if OS version leter than or equal to parameters.
+     */
+    static bool IsOSVersion(
+      unsigned major,     ///< Major version number
+      unsigned minor = 0, ///< Minor version number
+      unsigned build = 0  ///< Build number
+    );
+
     /**Get the configuration directory of the operating system the process is
        running on, eg "/etc" for Unix, "c:\windows" for Win95 or
        "c:\winnt\system32\drivers\etc" for NT.
@@ -578,7 +620,7 @@ class PProcess : public PThread
     PTimerList * GetTimerList();
 
     /**Internal initialisation function called directly from
-       #InternalMain()#. The user should never call this function.
+       <code>InternalMain()</code>. The user should never call this function.
      */
     void PreInitialise(
       int argc,     // Number of program arguments.
@@ -587,7 +629,7 @@ class PProcess : public PThread
     );
 
     /**Internal shutdown function called directly from the ~PProcess
-       #InternalMain()#. The user should never call this function.
+       <code>InternalMain()</code>. The user should never call this function.
      */
     static void PreShutdown();
     static void PostShutdown();
@@ -693,8 +735,11 @@ class PProcess : public PThread
 
     bool m_library;
 
-    PDictionary<POrdinalKey, PThread> activeThreads;
-    PMutex                            activeThreadMutex;
+    bool m_shuttingDown;
+
+    typedef std::map<PThreadIdentifier, PThread *> ThreadMap;
+    ThreadMap m_activeThreads;
+    PMutex    m_activeThreadMutex;
     
   friend class PThread;
 
@@ -722,11 +767,11 @@ class PProcess : public PThread
     PLibraryProcess(
       const char * manuf = "",         ///< Name of manufacturer
       const char * name = "",          ///< Name of product
-      WORD majorVersion = 1,           ///< Major version number of the product
-      WORD minorVersion = 0,           ///< Minor version number of the product
-      CodeStatus status = ReleaseCode, ///< Development status of the product
-      WORD buildNumber = 1             ///< Build number of the product
-    ) : PProcess(manuf, name, majorVersion, minorVersion, status, buildNumber, true) { }
+      WORD majorVersionNum = 1,           ///< Major version number of the product
+      WORD minorVersionNum = 0,           ///< Minor version number of the product
+      CodeStatus statusCode = ReleaseCode, ///< Development status of the product
+      WORD buildNum = 1             ///< Build number of the product
+    ) : PProcess(manuf, name, majorVersionNum, minorVersionNum, statusCode, buildNum, true) { }
   //@}
 
     ///< Dummy Main() as libraries do not have one.
