@@ -27,9 +27,9 @@
  *
  * Contributor(s): /
  *
- * $Revision: 23185 $
- * $Author: rjongbloed $
- * $Date: 2009-07-29 19:13:36 -0500 (Wed, 29 Jul 2009) $
+ * $Revision: 25459 $
+ * $Author: csoutheren $
+ * $Date: 2011-03-30 18:10:26 -0500 (Wed, 30 Mar 2011) $
  */
 
 #pragma implementation "sound_alsa.h"
@@ -67,14 +67,14 @@ PSoundChannelALSA::PSoundChannelALSA(const PString & device,
 
 void PSoundChannelALSA::Construct()
 {
-  enum _snd_pcm_format val;
-
-#if PBYTE_ORDER == PLITTLE_ENDIAN
-  val = (mBitsPerSample == 16) ? SND_PCM_FORMAT_S16_LE : SND_PCM_FORMAT_U8;
-#else
-  val = (mBitsPerSample == 16) ? SND_PCM_FORMAT_S16_BE : SND_PCM_FORMAT_U8;
-#endif
-
+//  enum _snd_pcm_format val;
+//
+//#if PBYTE_ORDER == PLITTLE_ENDIAN
+//  val = (mBitsPerSample == 16) ? SND_PCM_FORMAT_S16_LE : SND_PCM_FORMAT_U8;
+//#else
+//  val = (mBitsPerSample == 16) ? SND_PCM_FORMAT_S16_BE : SND_PCM_FORMAT_U8;
+//#endif
+//
   frameBytes = 2;
   m_bufferSize = 320; // 20 ms worth of 8kHz data
   m_bufferCount = 2;  // double buffering
