@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23936 $
+ * $Revision: 24412 $
  * $Author: rjongbloed $
- * $Date: 2010-01-13 20:36:07 -0600 (Wed, 13 Jan 2010) $
+ * $Date: 2010-05-27 18:53:50 -0500 (Thu, 27 May 2010) $
  */
 
 //////////////////////////////////////////////////
@@ -506,6 +506,9 @@ bool PMonitoredSockets::CreateSocket(SocketInfo & info, const PIPSocket::Address
     return true;
   }
 
+  PTRACE(1, "MonSock\tCould not listen on "
+         << binding << ':' << localPort
+         << " - " << info.socket->GetErrorText());
   delete info.socket;
   info.socket = NULL;
   return false;
@@ -630,7 +633,7 @@ PChannel::Errors PMonitoredSockets::ReadFromSocket(PSocket::SelectList & readers
 
     case EMSGSIZE :
       PTRACE(2, "MonSock\tRead UDP packet too large for buffer of " << len << " bytes.");
-      return PChannel::NoError;
+      return PChannel::BufferTooSmall;
 
     case EBADF : // Interface went down
     case EINTR :
@@ -1054,7 +1057,7 @@ PSingleMonitoredSocket::~PSingleMonitoredSocket()
 }
 
 
-PStringArray PSingleMonitoredSocket::GetInterfaces(PBoolean /*includeLoopBack*/, const PIPSocket::Address & /*destination*/)
+PStringArray PSingleMonitoredSocket::GetInterfaces(bool /*includeLoopBack*/, const PIPSocket::Address & /*destination*/)
 {
   PSafeLockReadOnly guard(*this);
 
