@@ -27,9 +27,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21446 $
+ * $Revision: 26303 $
  * $Author: rjongbloed $
- * $Date: 2008-11-04 07:20:54 +0000 (Tue, 04 Nov 2008) $
+ * $Date: 2011-08-12 03:44:54 -0500 (Fri, 12 Aug 2011) $
  */
 
 #ifdef __GNUC__
@@ -49,18 +49,12 @@ const char WAVLabelFMT_[4] = { 'f', 'm', 't', ' ' };
 const char WAVLabelFACT[4] = { 'F', 'A', 'C', 'T' };
 const char WAVLabelDATA[4] = { 'd', 'a', 't', 'a' };
 
-PINSTANTIATE_FACTORY(PWAVFileFormat, unsigned)
-PINSTANTIATE_FACTORY(PWAVFileConverter, unsigned)
 
 inline PBoolean ReadAndCheck(PWAVFile & file, void * buf, PINDEX len)
 {
   return file.FileRead(buf, len) && (file.PFile::GetLastReadCount() == len);
 }
 
-inline PBoolean WriteAndCheck(PWAVFile & file, void * buf, PINDEX len)
-{
-  return file.FileWrite(buf, len) && (file.GetLastWriteCount() == len);
-}
 
 #if PBYTE_ORDER==PBIG_ENDIAN
 #  if defined(USE_SYSTEM_SWAB)
@@ -87,20 +81,12 @@ static void SWAB(const void * void_from, void * void_to, register size_t len)
 // PWAVFile
 
 PWAVFile::PWAVFile(unsigned fmt)
-  : PFile(), origFmt(fmt)
+  : origFmt(fmt)
 {
   Construct();
   SelectFormat(fmt);
 }
 
-PWAVFile * PWAVFile::format(const PString & format)
-{
-  PWAVFile * file = new PWAVFile;
-  file->origFmt = 0xffffffff;
-  file->SelectFormat(format);
-  return file;
-}
-
 PWAVFile::PWAVFile(OpenMode mode, int opts, unsigned fmt)
   : PFile(mode, opts), origFmt(fmt)
 {
@@ -108,18 +94,6 @@ PWAVFile::PWAVFile(OpenMode mode, int opts, unsigned fmt)
   SelectFormat(fmt);
 }
 
-PWAVFile * PWAVFile::format(
-  const PString & format,
-  PFile::OpenMode mode,
-  int opts
-)
-{
-  PWAVFile * file = new PWAVFile(mode, opts);
-  file->origFmt = 0xffffffff;
-  file->SelectFormat(format);
-  return file;
-}
-
 PWAVFile::PWAVFile(const PFilePath & name, OpenMode mode, int opts, unsigned fmt)
   : origFmt(fmt)
 {
@@ -144,8 +118,9 @@ PWAVFile::PWAVFile(
 PWAVFile::~PWAVFile()
 { 
   Close(); 
-  if (formatHandler != NULL)
-    delete formatHandler;
+
+  delete formatHandler;
+  delete autoConverter;
 }
 
 
@@ -162,31 +137,56 @@ void PWAVFile::Construct()
   wavFmtChunk.hdr.len     = sizeof(wavFmtChunk) - sizeof(wavFmtChunk.hdr);
 }
 
-void PWAVFile::SelectFormat(unsigned fmt)
+
+PWAVFile * PWAVFile::format(const PString & format)
 {
-  if (formatHandler != NULL) {
-    delete formatHandler;
-    formatHandler = NULL;
-  }
-  if (fmt != fmt_NotKnown) {
-    formatHandler       = PWAVFileFormatByIDFactory::CreateInstance(fmt);
-    wavFmtChunk.format  = (WORD)fmt;
-  }
+  PWAVFile * file = new PWAVFile;
+  file->origFmt = 0xffffffff;
+  file->SelectFormat(format);
+  return file;
 }
 
-void PWAVFile::SelectFormat(const PString & format)
+PWAVFile * PWAVFile::format(const PString & format, PFile::OpenMode mode, int opts)
 {
-  if (formatHandler != NULL) {
-    delete formatHandler;
-    formatHandler = NULL;
-  }
-  if (!format.IsEmpty())
-    formatHandler = PWAVFileFormatByFormatFactory::CreateInstance(format);
-  if (formatHandler != NULL) {
-    wavFmtChunk.format = (WORD)formatHandler->GetFormat();
-    if (origFmt == 0xffffffff)
-      origFmt = wavFmtChunk.format;
-  }
+  PWAVFile * file = new PWAVFile(mode, opts);
+  file->origFmt = 0xffffffff;
+  file->SelectFormat(format);
+  return file;
+}
+
+
+bool PWAVFile::SelectFormat(unsigned fmt)
+{
+  delete formatHandler;
+  formatHandler = NULL;
+
+  if (fmt == fmt_NotKnown)
+    return true;
+
+  formatHandler = PWAVFileFormatByIDFactory::CreateInstance(fmt);
+  if (formatHandler == NULL)
+    return false;
+
+  wavFmtChunk.format  = (WORD)fmt;
+  return true;
+}
+
+bool PWAVFile::SelectFormat(const PString & format)
+{
+  delete formatHandler;
+  formatHandler = NULL;
+
+  if (format.IsEmpty())
+    return false;
+
+  formatHandler = PWAVFileFormatByFormatFactory::CreateInstance(format);
+  if (formatHandler == NULL)
+    return SelectFormat(format.AsUnsigned());
+
+  wavFmtChunk.format = (WORD)formatHandler->GetFormat();
+  if (origFmt == 0xffffffff)
+    origFmt = wavFmtChunk.format;
+  return true;
 }
 
 PBoolean PWAVFile::Open(OpenMode mode, int opts)
@@ -225,6 +225,7 @@ PBoolean PWAVFile::Open(OpenMode mode, int opts)
   // if we did not know the format when we opened, then we had better know it now
   if (formatHandler == NULL) {
     Close();
+    SetErrorValues(BadParameter, EINVAL);
     return PFalse;
   }
 
@@ -243,7 +244,8 @@ PBoolean PWAVFile::Open(const PFilePath & name, OpenMode mode, int opts)
 
 PBoolean PWAVFile::Close()
 {
-  autoConverter = NULL; // This is a pointer to a singleton, no need to delete
+  delete autoConverter;
+  autoConverter = NULL;
 
   if (!PFile::IsOpen())
     return PTrue;
@@ -256,6 +258,7 @@ PBoolean PWAVFile::Close()
 
   delete formatHandler;
   formatHandler = NULL;
+
   if (origFmt != 0xffffffff)
     SelectFormat(origFmt);
 
@@ -271,8 +274,8 @@ void PWAVFile::SetAutoconvert()
 // Performs necessary byte-order swapping on for big-endian platforms.
 PBoolean PWAVFile::Read(void * buf, PINDEX len)
 {
-	if (!IsOpen())
-		return PFalse;
+  if (!IsOpen())
+    return PFalse;
 
   if (autoConverter != NULL)
     return autoConverter->Read(*this, buf, len);
@@ -307,8 +310,8 @@ PBoolean PWAVFile::FileRead(void * buf, PINDEX len)
 // Performs necessary byte-order swapping on for big-endian platforms.
 PBoolean PWAVFile::Write(const void * buf, PINDEX len)
 {
-	if (!IsOpen())
-		return PFalse;
+  if (!IsOpen())
+    return PFalse;
 
   // Needs to update header on close.
   header_needs_updating = PTrue;
@@ -500,9 +503,7 @@ PBoolean PWAVFile::SetFormat(const PString & format)
   if (IsOpen() || isValidWAV)
     return PFalse;
 
-  SelectFormat(format);
-
-  return PTrue;
+  return SelectFormat(format);
 }
 
 static inline PBoolean NeedsConverter(const PWAV::FMTChunk & fmtChunk)
@@ -512,7 +513,8 @@ static inline PBoolean NeedsConverter(const PWAV::FMTChunk & fmtChunk)
 
 PBoolean PWAVFile::ProcessHeader() 
 {
-  autoConverter = NULL; // This is a pointer to a singleton, no need to delete
+  delete autoConverter;
+  autoConverter = NULL;
 
   // Process the header information
   // This comes in 3 or 4 chunks, either RIFF, FORMAT and DATA
@@ -616,7 +618,8 @@ PBoolean PWAVFile::ProcessHeader()
 
 PBoolean PWAVFile::GenerateHeader()
 {
-  autoConverter = NULL; // This is a pointer to a singleton, no need to delete
+  delete autoConverter;
+  autoConverter = NULL;
 
   if (!IsOpen()) {
     PTRACE(1, "WAV\tGenerateHeader: Not Open");
@@ -646,7 +649,7 @@ PBoolean PWAVFile::GenerateHeader()
   memcpy(riffChunk.tag,     WAVLabelWAVE, sizeof(WAVLabelWAVE));
   riffChunk.hdr.len = lenHeader + audioDataLen - sizeof(riffChunk.hdr);
 
-  if (!WriteAndCheck(*this, &riffChunk, sizeof(riffChunk)))
+  if (!FileWrite(&riffChunk, sizeof(riffChunk)))
     return PFalse;
 
   // populate and write the WAV header with the default data
@@ -661,11 +664,11 @@ PBoolean PWAVFile::GenerateHeader()
   formatHandler->CreateHeader(wavFmtChunk, extendedHeader);
 
   // write the basic WAV header
-  if (
-      !WriteAndCheck(*this, &wavFmtChunk, sizeof(wavFmtChunk)) ||
-      ((extendedHeader.GetSize() > 0) && !WriteAndCheck(*this, extendedHeader.GetPointer(), extendedHeader.GetSize()))
-     )
-    return PFalse;
+  if (!FileWrite(&wavFmtChunk, sizeof(wavFmtChunk)))
+    return false;
+
+  if (extendedHeader.GetSize() > 0 && !FileWrite(extendedHeader.GetPointer(), extendedHeader.GetSize()))
+    return false;
 
   // allow the format handler to write additional chunks
   if (!formatHandler->WriteExtraChunks(*this))
@@ -675,7 +678,7 @@ PBoolean PWAVFile::GenerateHeader()
   PWAV::ChunkHeader dataChunk;
   memcpy(dataChunk.tag, WAVLabelDATA, sizeof(WAVLabelDATA));
   dataChunk.len = audioDataLen;
-  if (!WriteAndCheck(*this, &dataChunk, sizeof(dataChunk)))
+  if (!FileWrite(&dataChunk, sizeof(dataChunk)))
     return PFalse;
 
   isValidWAV = PTrue;
@@ -716,13 +719,13 @@ PBoolean PWAVFile::UpdateHeader()
   // rewrite the length in the RIFF chunk
   PInt32l riffChunkLen = (lenHeader - 8) + lenData; // size does not include first 8 bytes
   PFile::SetPosition(4);
-  if (!WriteAndCheck(*this, &riffChunkLen, sizeof(riffChunkLen)))
+  if (!FileWrite(&riffChunkLen, sizeof(riffChunkLen)))
     return PFalse;
 
   // rewrite the data length field in the data chunk
   PInt32l dataChunkLen = lenData;
   PFile::SetPosition(lenHeader - 4);
-  if (!WriteAndCheck(*this, &dataChunkLen, sizeof(dataChunkLen)))
+  if (!FileWrite(&dataChunkLen, sizeof(dataChunkLen)))
     return PFalse;
 
   if(formatHandler == NULL){
@@ -732,10 +735,10 @@ PBoolean PWAVFile::UpdateHeader()
   formatHandler->UpdateHeader(wavFmtChunk, extendedHeader);
 
   PFile::SetPosition(12);
-  if (!WriteAndCheck(*this, &wavFmtChunk, sizeof(wavFmtChunk)))
+  if (!FileWrite(&wavFmtChunk, sizeof(wavFmtChunk)))
     return PFalse;
 
-  if (!WriteAndCheck(*this, extendedHeader.GetPointer(), extendedHeader.GetSize()))
+  if (!FileWrite(extendedHeader.GetPointer(), extendedHeader.GetSize()))
     return PFalse;
 
   header_needs_updating = PFalse;
@@ -785,7 +788,7 @@ class PWAVFileFormatPCM : public PWAVFileFormat
     PBoolean Write(PWAVFile & file, const void * buf, PINDEX & len);
 };
 
-PWAVFileFormatByIDFactory::Worker<PWAVFileFormatPCM> pcmIDWAVFormat(PWAVFile::fmt_PCM);
+PFACTORY_CREATE(PWAVFileFormatByIDFactory, PWAVFileFormatPCM, PWAVFile::fmt_PCM, false);
 PWAVFileFormatByFormatFactory::Worker<PWAVFileFormatPCM> pcmFormatWAVFormat("PCM-16");
 
 void PWAVFileFormatPCM::CreateHeader(PWAV::FMTChunk & wavFmtChunk, 
@@ -817,7 +820,7 @@ PBoolean PWAVFileFormatPCM::Read(PWAVFile & file, void * buf, PINDEX & len)
   // WAV files are little-endian. So swap the bytes if this is
   // a big endian machine and we have 16 bit samples
   // Note: swab only works on even length buffers.
-  if (file.wavFmtChunk.bitsPerSample == 16) {
+  if (file.GetSampleSize() == 16) {
     SWAB(buf, buf, len);
   }
 
@@ -829,7 +832,7 @@ PBoolean PWAVFileFormatPCM::Write(PWAVFile & file, const void * buf, PINDEX & le
   // WAV files are little-endian. So swap the bytes if this is
   // a big endian machine and we have 16 bit samples
   // Note: swab only works on even length buffers.
-  if (file.wavFmtChunk.bitsPerSample == 16) {
+  if (file.GetSampleSize() == 16) {
     SWAB(buf, (void *)buf, len);
   }
 
@@ -1079,11 +1082,11 @@ off_t PWAVFileConverterPCM::GetDataLength(PWAVFile & file)
 
 PBoolean PWAVFileConverterPCM::Read(PWAVFile & file, void * buf, PINDEX len)
 {
-  if (file.wavFmtChunk.bitsPerSample == 16)
+  if (file.GetSampleSize() == 16)
     return file.PWAVFile::RawRead(buf, len);
 
-  if (file.wavFmtChunk.bitsPerSample != 8) {
-    PTRACE(1, "PWAVFile\tAttempt to read autoconvert PCM data with unsupported number of bits per sample " << (int)file.wavFmtChunk.bitsPerSample);
+  if (file.GetSampleSize() != 8) {
+    PTRACE(1, "PWAVFile\tAttempt to read autoconvert PCM data with unsupported number of bits per sample " << file.GetSampleSize());
     return PFalse;
   }
 
@@ -1108,13 +1111,13 @@ PBoolean PWAVFileConverterPCM::Read(PWAVFile & file, void * buf, PINDEX len)
 
 PBoolean PWAVFileConverterPCM::Write(PWAVFile & file, const void * buf, PINDEX len)
 {
-  if (file.wavFmtChunk.bitsPerSample == 16)
+  if (file.GetSampleSize() == 16)
     return file.PWAVFile::RawWrite(buf, len);
 
-  PTRACE(1, "PWAVFile\tAttempt to write autoconvert PCM data with unsupported number of bits per sample " << (int)file.wavFmtChunk.bitsPerSample);
+  PTRACE(1, "PWAVFile\tAttempt to write autoconvert PCM data with unsupported number of bits per sample " << file.GetSampleSize());
   return PFalse;
 }
 
-PWAVFileConverterFactory::Worker<PWAVFileConverterPCM> pcmConverter(PWAVFile::fmt_PCM, true);
+PWAVFileConverterFactory::Worker<PWAVFileConverterPCM> pcmConverter(PWAVFile::fmt_PCM);
 
 //////////////////////////////////////////////////////////////////
