@@ -25,9 +25,9 @@
  *
  * Based on code originally by Bob Jenkins.
  *
- * $Revision: 21783 $
+ * $Revision: 23891 $
  * $Author: rjongbloed $
- * $Date: 2008-12-11 20:59:27 -0600 (Thu, 11 Dec 2008) $
+ * $Date: 2009-12-22 01:57:21 -0600 (Tue, 22 Dec 2009) $
  */
 
 
@@ -45,7 +45,7 @@
 
 PRandom::PRandom()
 {
-  SetSeed((DWORD)(time(NULL)+clock()));
+  SetSeed(PTimer::Tick().GetInterval());
 }
 
 
