@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25892 $
+ * $Revision: 26287 $
  * $Author: rjongbloed $
- * $Date: 2011-05-25 20:27:19 -0500 (Wed, 25 May 2011) $
+ * $Date: 2011-08-08 05:53:35 -0500 (Mon, 08 Aug 2011) $
  */
 
 #ifdef __GNUC__
@@ -2345,9 +2345,10 @@ PString PVXMLChannel::AdjustWavFilename(const PString & ofn)
 
 PWAVFile * PVXMLChannel::CreateWAVFile(const PFilePath & fn, PBoolean recording)
 { 
-  PWAVFile * wav = PWAVFile::format(mediaFormat);
-  if (wav == NULL) {
+  PWAVFile * wav = new PWAVFile;
+  if (!wav->SetFormat(mediaFormat)) {
     PTRACE(1, "VXML\tWAV file format " << mediaFormat << " not known");
+    delete wav;
     return NULL;
   }
 
