@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24357 $
+ * $Revision: 25554 $
  * $Author: rjongbloed $
- * $Date: 2010-05-18 02:52:29 -0500 (Tue, 18 May 2010) $
+ * $Date: 2011-04-13 03:34:50 -0500 (Wed, 13 Apr 2011) $
  */
 
 #include <ptlib.h>
@@ -1535,6 +1535,14 @@ PObject & PAbstractDictionary::GetRefAt(const PObject & key) const
 }
 
 
+void PAbstractDictionary::AbstractGetKeys(PArrayObjects & keys) const
+{
+  keys.SetSize(GetSize());
+  for (PINDEX i = 0; i < GetSize(); i++)
+    keys.SetAt(i, AbstractGetKeyAt(i).Clone());
+}
+
+
 void PAbstractDictionary::PrintOn(ostream &strm) const
 {
   char separator = strm.fill();
