@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24188 $
+ * $Revision: 26203 $
  * $Author: rjongbloed $
- * $Date: 2010-04-06 03:19:20 -0500 (Tue, 06 Apr 2010) $
+ * $Date: 2011-07-14 23:36:07 -0500 (Thu, 14 Jul 2011) $
  */
 
 #include <ptlib.h>
@@ -58,17 +58,25 @@ class PExternalThread : public PThread
     PExternalThread()
     {
       SetThreadName(PString::Empty());
-      PTRACE(5, "PTLib\tCreated external thread " << this << " for id " << GetCurrentThreadId());
+      PTRACE(5, "PTLib\tCreated external thread " << this << ", id " << GetCurrentThreadId());
     }
 
     ~PExternalThread()
     {
-      PTRACE(5, "PTLib\tDestroyed external thread " << this << " for id " << GetThreadId());
+#ifdef _WIN32
+      CleanUp();
+#endif
+      PTRACE(5, "PTLib\tDestroyed external thread " << this << ", id " << GetThreadId());
     }
 
-    void Main()
+    virtual void Main()
     {
     }
+
+    virtual void Terminate()
+    {
+      PTRACE(2, "PTLib\tCannot terminate external thread " << this << ", id " << GetThreadId());
+    }
 };
 
 
@@ -333,14 +341,21 @@ void PTrace::Initialise(unsigned level, const char * filename, const char * roll
   info.OpenTraceFile(filename);
 
 #if PTRACING
-  PProcess & process = PProcess::Current();
-  Begin(0, "", 0) << "\tVersion " << process.GetVersion(PTrue)
-                  << " by " << process.GetManufacturer()
-                  << " on " << process.GetOSClass() << ' ' << process.GetOSName()
-                  << " (" << process.GetOSVersion() << '-' << process.GetOSHardware()
-                  << ") with PTLib (v" << process.GetLibVersion()
-                  << ") at " << PTime().AsString("yyyy/M/d h:mm:ss.uuu")
-                  << End;
+  if (PProcess::IsInitialised ()) {
+    PProcess & process = PProcess::Current();
+    Begin(0, "", 0) << "\tVersion " << process.GetVersion(PTrue)
+                    << " by " << process.GetManufacturer()
+                    << " on " << PProcess::GetOSClass() << ' ' << PProcess::GetOSName()
+                    << " (" << PProcess::GetOSVersion() << '-' << PProcess::GetOSHardware()
+                    << ") with PTLib (v" << PProcess::GetLibVersion()
+                    << ") at " << PTime().AsString("yyyy/M/d h:mm:ss.uuu")
+                    << End;
+  } else  // allow to start tracing before PProcess creation, useful for program initialisation errors (e.g. opal codec loading)
+    Begin(0, "", 0) << " on " << PProcess::GetOSClass() << ' ' << PProcess::GetOSName()
+                    << " (" << PProcess::GetOSVersion() << '-' << PProcess::GetOSHardware()
+                    << ") with PTLib (v" << PProcess::GetLibVersion()
+                    << ") at " << PTime().AsString("yyyy/M/d h:mm:ss.uuu")
+                    << End;
 #endif
 
 }
@@ -631,8 +646,70 @@ void PDirectory::CloneContents(const PDirectory * d)
 
 
 ///////////////////////////////////////////////////////////////////////////////
-// PTimer
+// PSimpleTimer
 
+PSimpleTimer::PSimpleTimer(long milliseconds,
+                           int seconds,
+                           int minutes,
+                           int hours,
+                           int days)
+  : PTimeInterval(milliseconds, seconds, minutes, hours, days)
+  , m_startTick(PTimer::Tick())
+{
+}
+
+
+PSimpleTimer::PSimpleTimer(const PTimeInterval & time)
+  : PTimeInterval(time)
+  , m_startTick(PTimer::Tick())
+{
+}
+
+
+PSimpleTimer::PSimpleTimer(const PSimpleTimer & timer)
+  : PTimeInterval(timer)
+  , m_startTick(PTimer::Tick())
+{
+}
+
+
+PSimpleTimer & PSimpleTimer::operator=(DWORD milliseconds)
+{
+  PTimeInterval::operator=(milliseconds);
+  m_startTick = PTimer::Tick();
+  return *this;
+}
+
+
+PSimpleTimer & PSimpleTimer::operator=(const PTimeInterval & time)
+{
+  PTimeInterval::operator=(time);
+  m_startTick = PTimer::Tick();
+  return *this;
+}
+
+
+PSimpleTimer & PSimpleTimer::operator=(const PSimpleTimer & timer)
+{
+  PTimeInterval::operator=(timer);
+  m_startTick = PTimer::Tick();
+  return *this;
+}
+
+
+void PSimpleTimer::SetInterval(PInt64 milliseconds,
+                               long seconds,
+                               long minutes,
+                               long hours,
+                               int days)
+{
+  PTimeInterval::SetInterval(milliseconds, seconds, minutes, hours, days);
+  m_startTick = PTimer::Tick();
+}
+
+
+///////////////////////////////////////////////////////////////////////////////
+// PTimer
 
 PTimer::PTimer(long millisecs, int seconds, int minutes, int hours, int days)
   : m_resetTime(millisecs, seconds, minutes, hours, days)
@@ -1089,16 +1166,22 @@ PBoolean PArgList::Parse(const char * spec, PBoolean optionsBeforeParams)
   while (*spec != '\0') {
     if (*spec == '-')
       optionLetters += ' ';
-    else
+    else {
+      PAssert(optionLetters.Find(*spec) == P_MAX_INDEX, "Multiple occurrences of same option letter");
       optionLetters += *spec++;
+    }
+
     if (*spec == '-') {
       const char * base = ++spec;
       while (*spec != '\0' && *spec != '.' && *spec != ':' && *spec != ';')
         spec++;
-      optionNames[codeCount] = PString(base, spec-base);
+      PString newOpt(base, spec-base);
+      PAssert(optionNames.GetValuesIndex(newOpt) == P_MAX_INDEX, "Multiple occurrences of same option string");
+      optionNames[codeCount] = newOpt;
       if (*spec == '.')
         spec++;
     }
+
     if (*spec == ':' || *spec == ';') {
       canHaveOptionString.SetSize(codeCount+1);
       canHaveOptionString[codeCount] = *spec == ':' ? 2 : 1;
@@ -1559,7 +1642,7 @@ PProcess & PProcess::Current()
 {
   if (PProcessInstance == NULL) {
     cerr << "Catastrophic failure, PProcess::Current() = NULL!!\n";
-#if defined(_MSC_VER) && defined(_DEBUG) && !defined(_WIN32_WCE)
+#if defined(_MSC_VER) && defined(_DEBUG) && !defined(_WIN32_WCE) && !defined(_WIN64)
     __asm int 3;
 #endif
     _exit(1);
@@ -1863,7 +1946,7 @@ PThread * PThread::Current()
   if (it != process.m_activeThreads.end())
     return it->second;
 
-  return new PExternalThread;
+  return process.m_shuttingDown ? NULL : new PExternalThread;
 }
 
 
@@ -1895,7 +1978,7 @@ static void SetWinDebugThreadName(const char * threadName, DWORD threadId)
 
   __try
   {
-    RaiseException(0x406D1388, 0, sizeof(threadInfo)/sizeof(DWORD), (DWORD *)&threadInfo) ;
+    RaiseException(0x406D1388, 0, sizeof(threadInfo)/sizeof(DWORD), (const ULONG_PTR *)&threadInfo) ;
     // if not running under debugger exception comes back
   }
   __except(EXCEPTION_CONTINUE_EXECUTION)
@@ -2193,17 +2276,36 @@ void PReadWriteMutex::StartRead()
 }
 
 
+void PReadWriteMutex::InternalWait(PSemaphore & semaphore) const
+{
+#if PTRACING
+  if (semaphore.Wait(15000))
+    return;
+
+  ostream & trace = PTrace::Begin(1, __FILE__, __LINE__);
+  trace << "PTLib\tPossible deadlock in read/write mutex:\n";
+  for (std::map<PThreadIdentifier, Nest>::const_iterator it = m_nestedThreads.begin(); it != m_nestedThreads.end(); ++it)
+    trace << "  thread-id=" << it->first << ","
+              " readers=" << it->second.readerCount << ","
+              " writers=" << it->second.writerCount << '\n';
+  trace << PTrace::End;
+#endif
+
+  semaphore.Wait();
+}
+
+
 void PReadWriteMutex::InternalStartRead()
 {
   // Text book read only lock
 
   starvationPreventer.Wait();
-   readerSemaphore.Wait();
+   InternalWait(readerSemaphore);
     readerMutex.Wait();
 
      readerCount++;
      if (readerCount == 1)
-       writerSemaphore.Wait();
+       InternalWait(writerSemaphore);
 
     readerMutex.Signal();
    readerSemaphore.Signal();
@@ -2282,11 +2384,11 @@ void PReadWriteMutex::StartWrite()
 
   writerCount++;
   if (writerCount == 1)
-    readerSemaphore.Wait();
+    InternalWait(readerSemaphore);
 
   writerMutex.Signal();
 
-  writerSemaphore.Wait();
+  InternalWait(writerSemaphore);
 }
 
 
