@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20560 $
- * $Author: rjongbloed $
- * $Date: 2008-07-10 00:57:34 -0500 (Thu, 10 Jul 2008) $
+ * $Revision: 24459 $
+ * $Author: shorne $
+ * $Date: 2010-06-06 08:59:59 -0500 (Sun, 06 Jun 2010) $
  */
 
 #include <ptlib.h>
@@ -121,7 +121,7 @@ PChannelStreamBuffer::pos_type PChannelStreamBuffer::seekoff(off_type off, ios_b
   sync();
   if (PIsDescendant(channel, PFile)) {
     PFile * file = (PFile *)channel;
-    file->SetPosition(off, (PFile::FilePositionOrigin)dir);
+    file->SetPosition((off_t)off, (PFile::FilePositionOrigin)dir);
     return file->GetPosition();
   }
 
@@ -486,7 +486,7 @@ PBoolean PChannel::SendCommandString(const PString & command)
         break;
 
       case NextCharDelay : // Delay in send
-        PThread::Current()->Sleep(timeout);
+        PThread::Sleep(timeout);
         break;
 
       case NextCharWait : // Wait for reply
@@ -522,6 +522,16 @@ PBoolean PChannel::Shutdown(ShutdownValue)
 }
 
 
+bool PChannel::SetLocalEcho(bool /*localEcho*/)
+{
+  return IsOpen();
+}
+
+bool PChannel::FlowControl(const void * /*flowData*/)
+{
+    return false;
+}
+
 PChannel * PChannel::GetBaseReadChannel() const
 {
   return (PChannel *)this;
@@ -748,6 +758,13 @@ PBoolean PIndirectChannel::Shutdown(ShutdownValue value)
 }
 
 
+bool PIndirectChannel::SetLocalEcho(bool localEcho)
+{
+  PReadWaitAndSignal mutex(channelPointerMutex);
+  return readChannel != NULL && readChannel->SetLocalEcho(localEcho);
+}
+
+
 PString PIndirectChannel::GetErrorText(ErrorGroup group) const
 {
   if (readChannel != NULL)
@@ -909,6 +926,9 @@ PBoolean PFile::Close()
 
 PBoolean PFile::Read(void * buffer, PINDEX amount)
 {
+  if (!IsOpen())
+    return SetErrorValues(NotOpen, EBADF);
+
   flush();
 #ifdef WOT_NO_FILESYSTEM
   lastReadCount = 0;
@@ -921,6 +941,9 @@ PBoolean PFile::Read(void * buffer, PINDEX amount)
 
 PBoolean PFile::Write(const void * buffer, PINDEX amount)
 {
+  if (!IsOpen())
+    return SetErrorValues(NotOpen, EBADF);
+
   flush();
 #ifdef WOT_NO_FILESYSTEM
   lastWriteCount = amount;
@@ -941,6 +964,9 @@ PBoolean PFile::Open(const PFilePath & name, OpenMode  mode, int opts)
 
 off_t PFile::GetLength() const
 {
+  if (!IsOpen())
+    return -1;
+
 #ifdef WOT_NO_FILESYSTEM
   return 0;
 #else
@@ -954,6 +980,9 @@ off_t PFile::GetLength() const
 
 PBoolean PFile::IsEndOfFile() const
 {
+  if (!IsOpen())
+    return true;
+
   ((PFile *)this)->flush();
   return GetPosition() >= GetLength();
 }
@@ -964,6 +993,9 @@ PBoolean PFile::SetPosition(off_t pos, FilePositionOrigin origin)
 #ifdef WOT_NO_FILESYSTEM
   return PTrue;
 #else
+  if (!IsOpen())
+    return SetErrorValues(NotOpen, EBADF);
+
   return _lseek(GetHandle(), pos, origin) != (off_t)-1;
 #endif
 }
