@@ -24,9 +24,9 @@
  * Contributor(s): ______________________________________.
  *                 Walter H Whitlock (twohives@nc.rr.com)
  *
- * $Revision: 22039 $
- * $Author: dsandras $
- * $Date: 2009-02-11 13:07:07 -0600 (Wed, 11 Feb 2009) $
+ * $Revision: 24925 $
+ * $Author: rjongbloed $
+ * $Date: 2010-12-06 00:28:31 -0600 (Mon, 06 Dec 2010) $
  */
 
 #include <ptlib.h>
@@ -45,9 +45,6 @@
 #pragma comment(lib, "gdi32.lib")
 #endif
 
-namespace PWLibStupidLinkerHacks {
-   int loadVideoForWindowsStuff;
-};
     
 #include <vfw.h>
 
@@ -85,8 +82,6 @@ BOOL VFWAPI capGetDriverDescriptionA (WORD wDriverIndex, LPSTR lpszName,
 #endif // __MINGW32
 
 
-#define STEP_GRAB_CAPTURE 1
-
 
 /**This class defines a video input device.
  */
@@ -101,7 +96,8 @@ class PVideoInputDevice_VideoForWindows : public PVideoInputDevice
 
     /**Close the video input device on destruction.
       */
-    ~PVideoInputDevice_VideoForWindows() { Close(); }
+    ~PVideoInputDevice_VideoForWindows();
+
 
     /** Is the device a camera, and obtain video
      */
@@ -110,13 +106,6 @@ class PVideoInputDevice_VideoForWindows : public PVideoInputDevice
     virtual PStringArray GetDeviceNames() const
       { return GetInputDeviceNames(); }
 
-    /**Retrieve a list of Device Capabilities
-      */
-    static bool GetDeviceCapabilities(
-      const PString & /*deviceName*/, ///< Name of device
-      Capabilities * /*caps*/         ///< List of supported capabilities
-    ) { return false; }
-
     /**Open the device given the device name.
       */
     virtual PBoolean Open(
@@ -132,6 +121,13 @@ class PVideoInputDevice_VideoForWindows : public PVideoInputDevice
       */
     virtual PBoolean Close();
 
+    /**Retrieve a list of Device Capabilities
+      */
+    bool GetDeviceCapabilities(
+      Capabilities * /*caps*/         ///< List of supported capabilities
+    );
+    static PBoolean GetDeviceCapabilities(const PString & deviceName, Capabilities * capabilities);
+
     /**Start the video device I/O.
       */
     virtual PBoolean Start();
@@ -202,17 +198,12 @@ class PVideoInputDevice_VideoForWindows : public PVideoInputDevice
     );
 
 
-    /**Try all known video formats & see which ones are accepted by the video driver
-     */
-    virtual PBoolean TestAllFormats();
+    virtual bool SetCaptureMode(unsigned mode);
+    virtual int GetCaptureMode() const { return useVideoMode; }
+
 
   protected:
-
-   /**Check the hardware can do the asked for size.
-
-       Note that not all cameras can provide all frame sizes.
-     */
-    virtual PBoolean VerifyHardwareFrameSize(unsigned width, unsigned height);
+    PBoolean VerifyHardwareFrameSize(unsigned width, unsigned height);
 
     PDECLARE_NOTIFIER(PThread, PVideoInputDevice_VideoForWindows, HandleCapture);
 
@@ -229,7 +220,8 @@ class PVideoInputDevice_VideoForWindows : public PVideoInputDevice
     PMutex        operationMutex;
 
     PSyncPoint    frameAvailable;
-    LPBYTE        lastFramePtr;
+    bool          useVideoMode;
+    LPBYTE        lastFrameData;
     unsigned      lastFrameSize;
     PMutex        lastFrameMutex;
     bool          isCapturingNow;
@@ -406,11 +398,71 @@ PVideoInputDevice_VideoForWindows::PVideoInputDevice_VideoForWindows()
 {
   captureThread = NULL;
   hCaptureWindow = NULL;
-  lastFramePtr = NULL;
   lastFrameSize = 0;
   isCapturingNow = PFalse;
+
+  useVideoMode    = PFalse;
+  lastFrameData   = NULL;
 }
 
+
+PVideoInputDevice_VideoForWindows::~PVideoInputDevice_VideoForWindows()
+{
+  if(lastFrameData)
+    delete[] lastFrameData;
+  Close();
+}
+
+
+bool PVideoInputDevice_VideoForWindows::SetCaptureMode(unsigned mode)
+{
+  useVideoMode = mode != 0;
+
+  // Do nothing if we are currently capturing (we don't support switching between picture- and video-mode during a capture).
+  if(IsCapturing())
+    return false;
+
+  // Set the callback function for complete frames
+  BOOL result;
+  if (useVideoMode)
+    result = capSetCallbackOnVideoStream(hCaptureWindow, VideoHandler);
+  else
+    result = capSetCallbackOnFrame(hCaptureWindow, VideoHandler);
+
+  if (!result) {
+    lastError = ::GetLastError();
+    PTRACE(1, "PVidInp\tFailed to set callback on VfW - " << lastError);
+    return false;
+  }
+
+  CAPTUREPARMS parms;
+  memset(&parms, 0, sizeof(parms));
+  if (!capCaptureGetSetup(hCaptureWindow, &parms, sizeof(parms))) {
+    lastError = ::GetLastError();
+    PTRACE(1, "PVidInp\tcapCaptureGetSetup: failed - " << lastError);
+    return PFalse;
+  }
+
+  // For video mode we must tell VfW to work in a separate background thread, or our application will lock otherwise.
+  if (useVideoMode) {
+    parms.fYield = TRUE;
+    parms.dwIndexSize = 324000;
+  }
+  else {
+    parms.fYield = FALSE;
+    parms.dwIndexSize = 0;
+  }
+
+  if (!capCaptureSetSetup(hCaptureWindow, &parms, sizeof(parms))) {
+    lastError = ::GetLastError();
+    PTRACE(1, "PVidInp\tcapCaptureSetSetup: failed - " << lastError);
+    return false;
+  }
+
+  return true;
+}
+
+
 PBoolean PVideoInputDevice_VideoForWindows::Open(const PString & devName, PBoolean startImmediate)
 {
   Close();
@@ -477,6 +529,34 @@ PBoolean PVideoInputDevice_VideoForWindows::Close()
 }
 
 
+PBoolean PVideoInputDevice_VideoForWindows::GetDeviceCapabilities(const PString & deviceName,
+                                                                  Capabilities * capabilities)
+{
+  PVideoInputDevice_VideoForWindows instance;
+  return instance.Open(deviceName, false) && instance.GetDeviceCapabilities(capabilities);
+}
+
+
+bool PVideoInputDevice_VideoForWindows::GetDeviceCapabilities(Capabilities * caps)
+{
+  for (PINDEX prefFormatIdx = 0; FormatTable[prefFormatIdx].colourFormat != NULL; prefFormatIdx++) {
+    PVideoDeviceBitmap bi(hCaptureWindow, FormatTable[prefFormatIdx].bitCount); 
+    bi->bmiHeader.biCompression = FormatTable[prefFormatIdx].compression;
+    for (PINDEX prefResizeIdx = 0; prefResizeIdx < PARRAYSIZE(winTestResTable); prefResizeIdx++) {
+      bi->bmiHeader.biWidth = winTestResTable[prefResizeIdx].device_width;
+      bi->bmiHeader.biHeight = winTestResTable[prefResizeIdx].device_height;
+      if (bi.ApplyFormat(hCaptureWindow, FormatTable[prefFormatIdx]) && caps != NULL) {
+        PVideoFrameInfo frameInfo;
+        frameInfo.SetFrameSize(winTestResTable[prefResizeIdx].device_width,
+                               winTestResTable[prefResizeIdx].device_height);
+        caps->framesizes.push_back(frameInfo);
+      }
+    }
+  }
+  return !caps->framesizes.empty();
+}
+
+
 PBoolean PVideoInputDevice_VideoForWindows::Start()
 {
   PWaitAndSignal mutex(operationMutex);
@@ -484,20 +564,24 @@ PBoolean PVideoInputDevice_VideoForWindows::Start()
   if (IsCapturing())
     return PTrue;
 
-#if STEP_GRAB_CAPTURE
-  isCapturingNow = PTrue;
-  return capGrabFrameNoStop(hCaptureWindow);
-#else
+  if (!useVideoMode) {
+    isCapturingNow = true;
+    return capGrabFrameNoStop(hCaptureWindow);
+  }
+
   if (capCaptureSequenceNoFile(hCaptureWindow)) {
     PCapStatus status(hCaptureWindow);
     isCapturingNow = status.fCapturingNow;
+
+    // As initializing the camera takes some time, and video-mode runs in a background thread, we need to wait for the first frame here.
+    // Otherwise "GetFrameDataNoDelay" might time-out.
+    frameAvailable.Wait();
     return isCapturingNow;
   }
 
   lastError = ::GetLastError();
   PTRACE(1, "PVidInp\tcapCaptureSequenceNoFile: failed - " << lastError);
   return PFalse;
-#endif
 }
 
 
@@ -508,16 +592,18 @@ PBoolean PVideoInputDevice_VideoForWindows::Stop()
   if (!IsCapturing())
     return PFalse;
   isCapturingNow = PFalse;
-#if STEP_GRAB_CAPTURE
-  return IsOpen() && frameAvailable.Wait(1000);
-#else
+
+  // If using the picture mode, we just need to wait for the very next frame ...
+  if (!useVideoMode)
+    return IsOpen() && frameAvailable.Wait(1000);
+
+  // ... otherwise we need to explicitely stop capturing.
   if (capCaptureStop(hCaptureWindow))
     return PTrue;
 
   lastError = ::GetLastError();
   PTRACE(1, "PVidInp\tcapCaptureStop: failed - " << lastError);
   return PFalse;
-#endif
 }
 
 
@@ -604,6 +690,7 @@ PBoolean PVideoInputDevice_VideoForWindows::SetFrameRate(unsigned rate)
   // keep current (default) framerate if 0==frameRate   
   if (0 != frameRate)
     parms.dwRequestMicroSecPerFrame = 1000000 / frameRate;
+
   parms.fMakeUserHitOKToCapture = PFalse;
   parms.wPercentDropForError = 100;
   parms.fCaptureAudio = PFalse;
@@ -671,29 +758,6 @@ PBoolean PVideoInputDevice_VideoForWindows::SetFrameSize(unsigned width, unsigne
 }
 
 
-PBoolean PVideoInputDevice_VideoForWindows::TestAllFormats()
-{
-  PBoolean running = IsCapturing();
-  if (running)
-    Stop();
-
-  for (PINDEX prefFormatIdx = 0; FormatTable[prefFormatIdx].colourFormat != NULL; prefFormatIdx++) {
-    PVideoDeviceBitmap bi(hCaptureWindow, FormatTable[prefFormatIdx].bitCount); 
-    bi->bmiHeader.biCompression = FormatTable[prefFormatIdx].compression;
-    for (PINDEX prefResizeIdx = 0; prefResizeIdx < PARRAYSIZE(winTestResTable); prefResizeIdx++) {
-      bi->bmiHeader.biWidth = winTestResTable[prefResizeIdx].device_width;
-      bi->bmiHeader.biHeight = winTestResTable[prefResizeIdx].device_height;
-      bi.ApplyFormat(hCaptureWindow, FormatTable[prefFormatIdx]);
-    } // for prefResizeIdx
-  } // for prefFormatIdx
-
-  if (running)
-    return Start();
-
-  return PTrue;
-}
-
-
 //return PTrue if absolute value of height reported by driver 
 //  is equal to absolute value of current frame height AND
 //  width reported by driver is equal to current frame width
@@ -763,11 +827,11 @@ PBoolean PVideoInputDevice_VideoForWindows::GetFrameDataNoDelay(BYTE * buffer, P
 
   lastFrameMutex.Wait();
 
-  if (lastFramePtr != NULL) {
+  if (lastFrameData != NULL) {
     if (NULL != converter)
-      retval = converter->Convert(lastFramePtr, buffer, bytesReturned);
+      retval = converter->Convert(lastFrameData, buffer, bytesReturned);
     else {
-      memcpy(buffer, lastFramePtr, lastFrameSize);
+      memcpy(buffer, lastFrameData, lastFrameSize);
       if (bytesReturned != NULL)
         *bytesReturned = lastFrameSize;
       retval = true;
@@ -776,10 +840,8 @@ PBoolean PVideoInputDevice_VideoForWindows::GetFrameDataNoDelay(BYTE * buffer, P
 
   lastFrameMutex.Signal();
 
-#if STEP_GRAB_CAPTURE
-  if (isCapturingNow)
+  if (!useVideoMode && isCapturingNow)
     capGrabFrameNoStop(hCaptureWindow);
-#endif
 
   return retval;
 }
@@ -817,10 +879,28 @@ LRESULT PVideoInputDevice_VideoForWindows::HandleVideo(LPVIDEOHDR vh)
 {
   if ((vh->dwFlags&(VHDR_DONE|VHDR_KEYFRAME)) != 0) {
     lastFrameMutex.Wait();
-    lastFramePtr = vh->lpData;
-    lastFrameSize = vh->dwBytesUsed;
-    if (lastFrameSize == 0)
-      lastFrameSize = vh->dwBufferLength;
+
+    /**
+    * As in video mode VfW captures in background, and hence might override the buffer of the current frame,
+    * we must copy the frame's data into a separate buffer.
+    */
+
+    // If the size of the current frame is same as of the old ...
+    //    -> ... simply copy the data of the new frame into the buffer ...
+    if(lastFrameSize == vh->dwBytesUsed)
+      memcpy(lastFrameData, vh->lpData, lastFrameSize);
+    else {
+      // ... otherwise delete the old buffer ...
+      if (lastFrameSize)
+        delete[] lastFrameData;
+
+      // ... and allocate a new one.
+      lastFrameSize = vh->dwBytesUsed;
+      lastFrameData = new BYTE[lastFrameSize];
+
+      memcpy(lastFrameData, vh->lpData, lastFrameSize);
+    }
+
     lastFrameMutex.Signal();
     frameAvailable.Signal();
   }
@@ -845,13 +925,15 @@ PBoolean PVideoInputDevice_VideoForWindows::InitialiseCapture()
 
   capSetCallbackOnError(hCaptureWindow, ErrorHandler);
 
-#if STEP_GRAB_CAPTURE
-  if (!capSetCallbackOnFrame(hCaptureWindow, VideoHandler)) { //} balance braces
-#else
-  if (!capSetCallbackOnVideoStream(hCaptureWindow, VideoHandler)) {
-#endif
+  BOOL result = FALSE;
+  if (useVideoMode)
+    result = capSetCallbackOnVideoStream(hCaptureWindow, VideoHandler);
+  else
+    result = capSetCallbackOnFrame(hCaptureWindow, VideoHandler);
+
+  if (!result) {
     lastError = ::GetLastError();
-    PTRACE(1, "PVidInp\tcapSetCallbackOnVideoStream failed - " << lastError);
+    PTRACE(1, "PVidInp\tFailed to set callback on VfW - " << lastError);
     return PFalse;
   }
 
@@ -921,8 +1003,18 @@ PBoolean PVideoInputDevice_VideoForWindows::InitialiseCapture()
   capPreview(hCaptureWindow, PFalse);
 
 #if PTRACING
-  //if (PTrace::CanTrace(6))
-  //  TestAllFormats(); // list acceptable formats and frame resolutions for video capture driver
+  if (PTrace::CanTrace(6)) {
+    // Display log for every format set
+    for (PINDEX prefFormatIdx = 0; FormatTable[prefFormatIdx].colourFormat != NULL; prefFormatIdx++) {
+      PVideoDeviceBitmap bi(hCaptureWindow, FormatTable[prefFormatIdx].bitCount); 
+      bi->bmiHeader.biCompression = FormatTable[prefFormatIdx].compression;
+      for (PINDEX prefResizeIdx = 0; prefResizeIdx < PARRAYSIZE(winTestResTable); prefResizeIdx++) {
+        bi->bmiHeader.biWidth = winTestResTable[prefResizeIdx].device_width;
+        bi->bmiHeader.biHeight = winTestResTable[prefResizeIdx].device_height;
+        bi.ApplyFormat(hCaptureWindow, FormatTable[prefFormatIdx]);
+      }
+    }
+  }
 #endif
   
   return SetFrameRate(frameRate) && SetColourFormatConverter(colourFormat.IsEmpty() ? PString("YUV420P") : colourFormat);
@@ -1062,6 +1154,17 @@ class PVideoOutputDevice_Window : public PVideoOutputDeviceRGB
       int & y   // Y position of device surface
     ) const;
 
+    /**Set the position of the output device, where relevant. For devices such as
+       files, this does nothing. For devices such as Windows, this sets the
+       position of the window on the screen.
+       
+       Returns: TRUE if the position can be set.
+      */
+    virtual bool SetPosition(
+      int x,  // X position of device surface
+      int y   // Y position of device surface
+    );
+
     LRESULT WndProc(UINT uMsg, WPARAM wParam, LPARAM lParam);
 
   protected:
@@ -1324,6 +1427,22 @@ PBoolean PVideoOutputDevice_Window::GetPosition(int & x, int & y) const
 }
 
 
+bool PVideoOutputDevice_Window::SetPosition(int x, int y)
+{
+  if (m_hWnd != NULL) {
+    RECT rect;
+    rect.top = y;
+    rect.left = x;
+    rect.bottom = y;
+    rect.right = x;
+    ::AdjustWindowRectEx(&rect, GetWindowLong(m_hWnd, GWL_STYLE), false, GetWindowLong(m_hWnd, GWL_EXSTYLE));
+    ::SetWindowPos(m_hWnd, HWND_TOP, x+(x-rect.left), y, 0, 0, SWP_NOSIZE);
+  }
+
+  return true;
+}
+
+
 void PVideoOutputDevice_Window::Draw(HDC hDC)
 {
   RECT rect;
