@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
- * $Author: rjongbloed $
- * $Date: 2008-06-04 05:40:38 -0500 (Wed, 04 Jun 2008) $
+ * $Revision: 25853 $
+ * $Author: csoutheren $
+ * $Date: 2011-05-21 16:27:55 -0500 (Sat, 21 May 2011) $
  */
 
 #pragma implementation "pipechan.h"
@@ -47,12 +47,17 @@
 #include <termio.h>
 #endif
 
-#if defined(P_MACOSX)
-#include <crt_externs.h>
-#endif
-
 #include "../common/pipechan.cxx"
 
+#if defined(P_MACOSX) && !defined(P_IPHONEOS)
+#  include <crt_externs.h>
+#  define environ (*_NSGetEnviron())
+#elif defined(P_SOLARIS) || defined(P_FREEBSD) || defined(P_OPENBSD) || defined (P_NETBSD) || defined(__BEOS__) || defined(P_MACOS) || defined (P_AIX) || defined(P_IRIX) || defined(P_QNX)
+extern char ** environ;
+#elif defined(P_IPHONEOS)
+char ** environ = NULL;
+#endif
+
 
 int PX_NewHandle(const char *, int);
 
@@ -106,6 +111,19 @@ PBoolean PPipeChannel::PlatformOpen(const PString & subProgram,
     PX_NewHandle("PPipeChannel stderrChildPipe", PMAX(stderrChildPipe[0], stderrChildPipe[1]));
   }
 
+  // Set up new environment if one specified.
+  char ** exec_environ = environ;
+  if (environment != NULL || !searchPath) {
+    exec_environ = (char **)calloc(environment->GetSize()+1, sizeof(char*));
+    for (PINDEX i = 0; i < environment->GetSize(); i++) {
+      PString key(environment->GetKeyAt(i));
+      if (searchPath || (key != "PATH")) {
+        PString str = key + '=' + environment->GetDataAt(i);
+        exec_environ[i] = strdup(str);
+      }
+    }
+  }
+
   // fork to allow us to execute the child
 #if defined(__BEOS__) || defined(P_IRIX)
   childPid = fork();
@@ -131,6 +149,9 @@ PBoolean PPipeChannel::PlatformOpen(const PString & subProgram,
       ::close(stderrChildPipe[1]);
       stderrChildPipe[1] = -1;
     }
+
+    if (exec_environ != environ)
+      free(exec_environ);
  
     os_handle = 0;
     return PTrue;
@@ -142,14 +163,16 @@ PBoolean PPipeChannel::PlatformOpen(const PString & subProgram,
   // is redirected
   if (toChildPipe[0] != -1) {
     ::close(STDIN_FILENO);
-    ::dup(toChildPipe[0]);
+    if (::dup(toChildPipe[0]) == -1)
+      return false;
     ::close(toChildPipe[0]);
     ::close(toChildPipe[1]);  
   } else {
     int fd = open("/dev/null", O_RDONLY);
     PAssertOS(fd >= 0);
     ::close(STDIN_FILENO);
-    ::dup(fd);
+    if (::dup(fd) == -1)
+      return false;
     ::close(fd);
   }
 
@@ -157,25 +180,30 @@ PBoolean PPipeChannel::PlatformOpen(const PString & subProgram,
   // and stderr is redirected
   if (fromChildPipe[1] != -1) {
     ::close(STDOUT_FILENO);
-    ::dup(fromChildPipe[1]);
+    if (::dup(fromChildPipe[1]) == -1)
+      return false;
     ::close(STDERR_FILENO);
     if (!stderrSeparate)
-      ::dup(fromChildPipe[1]);
+      if (::dup(fromChildPipe[1]) == -1)
+        return false;
     ::close(fromChildPipe[1]);
     ::close(fromChildPipe[0]); 
   } else if (mode != ReadWriteStd) {
     int fd = ::open("/dev/null", O_WRONLY);
     PAssertOS(fd >= 0);
     ::close(STDOUT_FILENO);
-    ::dup(fd);
+    if (::dup(fd) == -1)
+      return false;
     ::close(STDERR_FILENO);
     if (!stderrSeparate)
-      ::dup(fd);
+      if (::dup(fd) == -1)
+        return false;
     ::close(fd);
   }
 
   if (stderrSeparate) {
-    ::dup(stderrChildPipe[1]);
+    if (::dup(stderrChildPipe[1]) == -1)
+      return false;
     ::close(stderrChildPipe[1]);
     ::close(stderrChildPipe[0]); 
   }
@@ -197,27 +225,8 @@ PBoolean PPipeChannel::PlatformOpen(const PString & subProgram,
   for (i = 0; i < argumentList.GetSize(); i++) 
     args[i+1] = strdup(argumentList[i].GetPointer());
 
-  // Set up new environment if one specified.
-  if (environment != NULL) {
-#if defined(P_SOLARIS) || defined(P_FREEBSD) || defined(P_OPENBSD) || defined (P_NETBSD) || defined(__BEOS__) || defined(P_MACOSX) || defined(P_MACOS) || defined (P_AIX) || defined(P_IRIX) || defined(P_QNX)
-    extern char ** environ;
-#  if defined(P_MACOSX)
-#    define environ (*_NSGetEnviron())
-#  endif
-#  define __environ environ
-#endif
-    __environ = (char **)calloc(environment->GetSize()+1, sizeof(char*));
-    for (i = 0; i < environment->GetSize(); i++) {
-      PString str = environment->GetKeyAt(i) + '=' + environment->GetDataAt(i);
-      __environ[i] = strdup(str);
-    }
-  }
-
-  // execute the child as required
-  if (searchPath)
-    execvp(subProgram, args);
-  else
-    execv(subProgram, args);
+  // run the program
+  execve(subProgram, args, exec_environ);
 
   _exit(2);
   return PFalse;
@@ -227,6 +236,8 @@ PBoolean PPipeChannel::PlatformOpen(const PString & subProgram,
 
 PBoolean PPipeChannel::Close()
 {
+  bool wasRunning = false;
+
   // close pipe from child
   if (fromChildPipe[0] != -1) {
     ::close(fromChildPipe[0]);
@@ -262,7 +273,9 @@ PBoolean PPipeChannel::Close()
 
   // kill the child process
   if (IsRunning()) {
-    kill (childPid, SIGKILL);
+    wasRunning = true;
+    PTRACE(4, "PipeChannel\tChild being sent SIGKILL");
+    kill(childPid, SIGKILL);
     WaitForTermination();
   }
 
@@ -270,7 +283,7 @@ PBoolean PPipeChannel::Close()
   os_handle = -1;
   childPid  = 0;
 
-  return PTrue;
+  return wasRunning;
 }
 
 PBoolean PPipeChannel::Read(void * buffer, PINDEX len)
@@ -435,7 +448,8 @@ int PPipeChannel::WaitForTermination(const PTimeInterval & timeout)
 
 PBoolean PPipeChannel::Kill(int killType)
 {
-  return ConvertOSError(kill (childPid, killType));
+  PTRACE(4, "PipeChannel\tChild being sent signal " << killType);
+  return ConvertOSError(kill(childPid, killType));
 }
 
 PBoolean PPipeChannel::CanReadAndWrite()
